/* This file is part of the Allegria project
   Copyright (C) 2004 Francois Schiettekatte <francois.schiettekatte@umontreal.ca>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

// ExperimentalSetupImpl.h: interface for the ExperimentalSetupImpl class.
//
//////////////////////////////////////////////////////////////////////

#ifndef EXPERIMENTALSETUPIMPL_H
#define EXPERIMENTALSETUPIMPL_H

#include <qcombobox.h>
#include <qpushbutton.h>
#include <qvalidator.h>

#include "ElementParam.h"
#include "ExperimentalSetup.h"
#include "../XYQt/PeriodicTable.h"



class ExperimentalSetupImpl : public ExperimentalSetupDialog  
{
    Q_OBJECT

public:
	ExperimentalSetupImpl(QWidget * parent = 0, const char* name = 0);
	virtual ~ExperimentalSetupImpl();

	Atom beamElement;
	Atom monitorElement;

	ElementParam elemParam[MAXELEMENT];
	//QTabWidget* tabElementList;	// tab widget that contains each element parameter page

	Atom selectElement();
	Atom setElement(QPushButton* pb, QComboBox * cb = NULL);

	Atom findElement(int atomNum, QPushButton* pb, double mass = 0., QComboBox * cb = NULL);
	Atom findElement(QString atomName, QPushButton* pb);


public slots:
	void setBeam();
	void setMonitor();
	void readFromFile();
	void updateParams();
	void selectDir();
	bool saveParameterFiles();
	bool saveParameterFiles(QString filename);
    bool saveParameterFilesAs();

private:
	long numElem;
	ElementParamWidget *elementParamWidgetList[30]; // poiter to element parameter pages
	QSize ElementParamWidgetSize;

	bool load(QString filename);
	bool loadParams();
	void setBeam(int Z, double mass);
	void setMonitor(QString atomName);
	void setMonitor(int Z);

protected slots:
	void addElement();
	void removeElement();
	void setTextUnits(bool);
};

#endif // EXPERIMENTALSETUPIMPL_H
