
C=====================================================================
C     erd.f, version 2003  2003/11/04
C     reworked by Francois Schiettekatte, Universit de Montral
C
C     Creators and main contributors to the previous versions of this
C     program include: J.P. Matin, Ken Oxorn, S. Bultena
C
C     Distributed under the GNU Public Licence
C     ========================================
C
C     The program contained in this file may be used, distributed and
C     modified without limitation provided that the new program contains
C     this notice and is distributed according to the GNU Public Licence
C     with the following two provision:

C  1) To calculate dE/dx, the program calls a subroutine contained in
C     the file stop96.f, which is a simplified Fortran version of the
C     STOP96.BAS program. This subroutine reads parameter tables from
C     SRIM2000. That section of the code is thus covered by the
C     following copyright: (C) 1995, J.F.Ziegler, Yorktown,NY,10598.
C     and subject to authors use and copying policy

C  2) publications including data that have been analysed or converted
C     using this program must make reference to the following article:
C     "F. Schiettekatte, M. Chicoine, S. Gujrathi, P. Wei and K. Oxorn,
C      Allegria: a new interface to the ERD program, accepted for
C      publication in the proceedings of the 16th International
C      Conference on Ion Beam Analysis, Albuquerque, NM (2003)"
C
C     Purpose
C     =======
C     This program reads a command file (prepared by AlegraToF),
C     and iteratively converts mass separated energy spectra into depth
C     profiles, using Bragg's Rule for dE/dx calculation in compounds.
C
C
C=====================================================================
C
      PROGRAM ERD
C
C=====================================================================
C     ***** VERY IMPORTANT NOTE ****
C     The Rutherford cross section for recoiled Hydrogren from Helium
C     is not valid for E > 1MeV.  See FUNCTION SIG
C     ****************************************************************
C     *** THE  CROSS-SECTION USED IS ONLY VALID FOR A TOTAL ANGLE ****
C     *** OF 30 DEGREES                                           ****
C     ****************************************************************
C
C=====================================================================
C
C              *COMMON BLOCKS*
C  /C/
C      RETA1...... angle of incidence (theta-in) [radians]
C      RETA2M..... monitor detector angle [radians]
C      RETA2D(30). exit angle of detected element [radians]
C      P1......... step size (PASN) corrected for RETA1 [ug/cm2]
C      P2M........ step size corrected for RETA2M [ug/cm2]
C      P2D(30).... step size corrected for RETA2D [ug/cm2]
C      PASN....... step size [ug/cm2]
C      EIN........ energy of projectile [MeV]
C      EFIN....... lower energy limit [MeV]
C
C  /TADEDX/
C      ITABLE(5,30). table containing names of DEDX tables [*4]
C                     (1,I) : projectile entry thru element(I)
C                     (2,I) : detected monitor exit thru element(I)
C                     (3,I) : detected element exit thru element(I)
C                     (4,I) : detected element thru absorber
C                     (5,I) : detected monitor thru absorber
C      ZTABLE(2,5,30). table containing Z1, Z2 correspoinding to ITABLE elements
C      MTABLE(5,30).   table containing M1 correspoinding to Z1 in ITABLE elements
C
C  /SPEC/
C      IMON....... name of monitor energy spectrum [*1022]
C      IDET(30)... name of element energy spectrum [*1022]
C      IDER(30)... radical name of element energy spectrum [*1022]
C      IPRO(30)... name of profile spectra for element (I) [*1022]
C
C  /D/
C      MP......... mass of projectile [amu]
C      MCM........ mass of monitor [amu]
C      MDM........ mass of detected monitor (MCM if ERD, MP if RBS)
C      MCD........ mass of target element [amu]
C      MDD........ mass of detected element (MP if RBS, MCD if ERD)
C      IZP........ Z of projectile
C      IZM........ Z of monitor
C       IZDM....... Z of detected monitor (IZP if RBS, IZM if ERD)
C      IZD........ Z of target element
C      IZDD....... Z of detected element (IZP if RBS, IZD if ERD)
C
C  /N/
C      FKM......... monitor kinematic factor
C      ESUPM1...... upper energy for monitor [MeV] (corrected for
C      EINFM1...... lower energy for monitor [MeV]  .. absorber)
C
C  /IDE/ 
C      IDECON(1024,30).. profile spectra for target elements
C      MCDET(30)........ mass of target elements [amu]
C
C  /EL/
C       NELEM....... number of elements
C      TOFELE...... number of elements in specified target profile
C
C  /Q/
C      ROMEGA(30).. ratio of solid angles between monitor & element
C                    detectors
C      DXABS(30)... absorber for detected elements
C
C  /M/
C      R(30)....... normalization constant
C      ERMON....... average energy of monitor window (0th iteration)[MeV]
C      DXMON....... thickness of monitor window (0th iteration) [ug/cm2]
C      ED(30)...... energy/channel for element(I)
C      BD(30)...... intercept for element(I)
C      EM.......... energy/channel for monitor
C      BM.......... intercept for monitor
C
C  /IOS/
C      IOS......... status flag
C
C  /MONITOR/
C      MINF........ lower depth channel for monitor depth
C      MSUP........ upper depth channel for monitor depth
C
C  /EDGE/
C      EDGE(30).... depth limit of profile for element(I) [ug/cm2]
C      ENDAV(30)... calculated average of element with depth > EDGE
C      CUT(30)..... predefined depth limit of element(I) [ug/cm2]
C
C  /IT/
C      ERMONIT..... average energy of monitor (iterated)
C      DXMONIT..... average depth of monitor (iterated)
C
C  /CAL/
C       ECAL(1024,30).... Energy calibration for element(I)
C                         The value of ECAL is the UPPER edge of the
C                         channel
C       ECALM(1024)...... Energy calibration of monitor
C       ICHN_SURF(30)..... Surface channel for element (I)
C      ARHOD(30)........ Constant for PHD calculation for element(I)
C      ARHOM............ Constant for PHD calculation for monitor
C       ALPHA_SLOPE...... Slope of alpha particle energy calibration
C---------------------------------------------------------------------
C
C       RECDET(30):  recoil detected
C       DXABSM:      monitor absorber thickness
C       ROMEGAM:     ratio of monitor solid angle
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      CHARACTER*2 RECMON,RECDET,PROJ,RECPAR, ABSELEM
      CHARACTER*3 EXTP,TYPE(30)
      CHARACTER*4 ITABLE
      CHARACTER*6 TARPRO
      CHARACTER*1022 IDET, IDER, IMON
      CHARACTER*1022 IPRO
      INTEGER ZTABLE, DPUN

      DOUBLE PRECISION  MTABLE, SURFE

      DIMENSION ITABLE(5,30),IPRO(30),FKD(30),RECDET(30),IZDET(30),
     1         ED(30),BD(30),R(30),DXABS(30),RETA2D(30),P2D(30),
     2         ROMEGA(30),PART(30),IDET(30), IDER(30)
      DIMENSION ECAL(1024,30),ECALM(1024),ARHOD(30),ICHN_SURF(30)
      DIMENSION ZTABLE(2,5,30), MTABLE(5,30)

      DOUBLE PRECISION  DXABSM, MP,MCM,MDM,MCD,MDD
        DOUBLE PRECISION  MCDET(30),MDDET(30),COMP(30,15)
        DOUBLE PRECISION  ARHOD,ARHOM,ALPHA_SLOPE

        DOUBLE PRECISION  SCOEF(93,54)


      INTEGER*4 IDECON(1024,30),TOFELE,BSUP,BINF,EDGE(30),ENDAV(30)
      INTEGER*4 CUT(30),INF(15),SUP(15)

        character*1024 WORKDIR, LOGFILE, SAMPDESC
        character*10 UNITDEPTH(2)

      COMMON/DSK/WORKDIR,PASN2
C
C
      COMMON/C/RETA1,RETA2M,RETA2D,P1,P2M,P2D,PASN,EIN,EFIN
      COMMON/TADEDX/ITABLE, ZTABLE, MTABLE !!,ITAB
        COMMON/SCOEF96/SCOEF,DPUN
      COMMON/SPEC/IMON,IDET,IPRO
      COMMON/D/MP,MCM,MDM,MCD,MDD,IZP,IZM,IZDM,IZD,IZDD
      COMMON/IDE/IDECON,MCDET
      COMMON/EL/NELEM,TOFELE
      COMMON/Q/ROMEGA,DXABS
      COMMON/M/R,ERMON,DXMON,ED,BD,EM,BM
      COMMON/EDGE/EDGE,ENDAV,CUT
      COMMON/IT/ERMONIT,DXMONIT
      COMMON/CAL/ECAL,ECALM,ICHN_SURF,ARHOD,ARHOM,ALPHA_SLOPE
C
C       set percentage of work done to 0
        CLOSE(3)
        OPEN(3, FILE='PROGRESS.ERD')
        WRITE(3,*) 0
        CLOSE(3)

        UNITDEPTH(1) = 'ug/cm2'
        UNITDEPTH(2) = 'E16 at/cm2'

        CALL STOPCOEF

        do 112 j=1,30
          do 112 i=1,5
             ZTABLE(1,i,j) = 0
             ZTABLE(2,i,j) = 0
             MTABLE(i,j) = 0.0d0
 112         ITABLE(i,j)= "    "

        READ(5,'(A1000)') WORKDIR
        READ(5,'(A1000)') LOGFILE
        READ(5,'(A1000)') SAMPDESC

        LOGFILE = WORKDIR(1:Len_Trim(WORKDIR)) //
     1                LOGFILE(1:Len_Trim(LOGFILE))

      open(11, FILE=LOGFILE)
      write(11,*) 'this is the 2003 version of ERD.for'
      write(11,*)
      write( 6,*) 'this is the 2003 version of ERD.for'
      write( 6,*)

      WRITE(11,*) 'Working dir: '// WORKDIR(1:Len_Trim(WORKDIR))
      WRITE(6,*) 'Working dir: '// WORKDIR(1:Len_Trim(WORKDIR))
      WRITE(11,*)'Sample description: '//SAMPDESC(1:Len_Trim(SAMPDESC))
      WRITE(6,*)'Sample description: '//SAMPDESC(1:Len_Trim(SAMPDESC))
      write(11,*)
      write(6,*)
C
      READ(5,*) PROJ,MP,IZP,TETAIN,EIN,EFIN,PASN,ALPHA_SLOPE, DPUN
      write(11,51)'Projectile:', PROJ
      write(11,52)'Projectile mass:', MP, 'amu'
      write(11,53)'Projectile Z:', IZP
      write(11,52)'Incident angle:', TETAIN, 'deg'
      write(11,52)'Projectile Energy:', EIN, 'MeV'
      write(11,52)'Conversion E limit:', EFIN, 'MeV'
      write(11,52)'Step size:', PASN, UNITDEPTH(DPUN+1)
      write(11,52)'alpha slope:', ALPHA_SLOPE, 'keV/ch'
      write(11,52)'depth units:', UNITDEPTH(DPUN+1)
      write(11,*)
      write(6,51)'Projectile:', PROJ
      write(6,52)'Projectile mass:', MP, 'amu'
      write(6,53)'Projectile Z:', IZP
      write(6,52)'Incident angle:', TETAIN, 'deg'
      write(6,52)'Projectile Energy:', EIN, 'MeV'
      write(6,52)'Conversion E limit:', EFIN, 'MeV'
      write(6,52)'Step size:', PASN, UNITDEPTH(DPUN+1)
      write(6,52)'alpha slope:', ALPHA_SLOPE, 'keV/ch'
      write(6,*)
 51   FORMAT(1X, A20, 1X, A2)
 52   FORMAT(1X, A20, F10.6, 1X, A)
 53   FORMAT(1X, A20, I3)

C
C --- Convert alpha slope to MeV
      ALPHA_SLOPE = ALPHA_SLOPE/1000.0d0

      PASN2 = PASN ! PASN2 is used in DISQUE function
C
C --- Monitor data
      READ(5,*) RECMON, MCM, IZM, RETA2M, ROMEGAM,
     1         ABSELEM, DXABSM, EM, BM, ARHOM, EXTP, BINF, BSUP, IMON
       write(6,*) 'ABSELEM=',ABSELEM,'*'
        ITABLE(5,1)(3:4) = ABSELEM(1:2)
        ZTABLE(2,5,1) = IZELEM(ABSELEM(1:2))

       write(11,*)'Monitor:'
       write(11,*)'EL Mass(amu) Z angle(deg) ratio Abs dx(' //
     1  UNITDEPTH(DPUN+1) //
     2  ') Slope(keV/ch) Zero(keV) Arho   Det ChLow ChHigh SpectrumFile'
       write(11,61) RECMON, MCM, IZM, RETA2M, ROMEGAM,
     1   ITABLE(5,1)(3:4), DXABSM, EM, BM, ARHOM, EXTP, BINF, BSUP,
     2   IMON(1:Len_Trim(IMON))
        write(11,*)

       write(6,*)'Monitor:'
       write(6,*)'EL Mass(amu) Z angle(deg) ratio Abs dx(' //
     1  UNITDEPTH(DPUN+1) //
     2  ') Slope(keV/ch) Zero(keV) Arho   Det ChLow ChHigh SpectrumFile'

       write(6,61) RECMON, MCM, IZM, RETA2M, ROMEGAM,
     1   ITABLE(5,1)(3:4), DXABSM, EM, BM, ARHOM, EXTP, BINF, BSUP,
     2   IMON(1:Len_Trim(IMON))
        write(6,*)

 61    FORMAT(1X,A2,F8.2,I4,F8.2,3X,F6.3,1X,A2,2X,F7.2,1X,F12.3,2X,
     1 F12.3,1X,F7.3,1X, A3,2I6,2X,A)

C
C --- Detected element data
       write(11,*)'Detected elements:'
       write(6,*)'Detected elements:'
       write(11,*)'#  EL Mass(amu) Z angle(deg) ratio Abs dx(' //
     1  UNITDEPTH(DPUN+1) //
     2   ') Slope(keV/ch) Zero(keV) Arho    Det SpectrumFile'
       write(6,*)'#  EL Mass(amu) Z angle(deg) ratio Abs dx(' //
     1  UNITDEPTH(DPUN+1) //
     2   ') Slope(keV/ch) Zero(keV) Arho    Det SpectrumFile'

      READ(5,*) NELEM
      DO 1350 IK = 1,NELEM
         READ(5,*) RECDET(IK), MCDET(IK), IZDET(IK),
     1         RETA2D(IK), ROMEGA(IK), ABSELEM, DXABS(IK),
     2         ED(IK), BD(IK), ARHOD(IK),TYPE(IK),IDER(IK)
           ITABLE(4,IK)(3:4) = ABSELEM
           ZTABLE(2,4,IK) = IZELEM(ABSELEM(1:2))

         write(11,62) IK,RECDET(IK), MCDET(IK), IZDET(IK), RETA2D(IK),
     1         ROMEGA(IK),ABSELEM, DXABS(IK),ED(IK), BD(IK),
     2         ARHOD(IK), TYPE(IK), IDER(IK)(1:Len_Trim(IDER(IK)))
         write(6,62) IK,RECDET(IK), MCDET(IK), IZDET(IK), RETA2D(IK),
     1         ROMEGA(IK),ABSELEM, DXABS(IK),ED(IK), BD(IK),
     2         ARHOD(IK), TYPE(IK), IDER(IK)(1:Len_Trim(IDER(IK)))

 62   FORMAT(1X,I2,1X,A2,F8.2,I4,F8.2,3X,F6.3,1X,A2,2X,F7.2,1X,F12.3,2X,
     1 F12.3,1X,F7.3,1X,A3,2X,A)

1350    CONTINUE

C --- Flush log file buffered data to disk
        CALL Flush(11)
C
C --- Parameters iterations based on predetermined comositions
      READ(5,*) NIT, TARPRO, TOFELE, NSTRATA
      write(11,*) NIT, ' iterations'
        write(11,*) 'TARPRO = ', TARPRO
        write(11,*) 'TOFELE = ', TOFELE, ' NSTRATA=', NSTRATA
      write(6,*) NIT, ' iterations'
        write(6,*) 'TARPRO = ', TARPRO
        write(6,*) 'TOFELE = ', TOFELE, ' NSTRATA=', NSTRATA

      IF(NIT .EQ. -1) THEN
        DO 1351 IK = NELEM+1, NELEM+TOFELE
1351        READ(5,*) IPRO(IK), MCDET(IK), EDGE(IK), ENDAV(IK), CUT(IK)
      ELSEIF (NIT .EQ. -2) THEN
        DO 1352 IK = NELEM+1, NELEM+TOFELE
        EDGE(IK) = 1024  ! DEDXIT ignores edge correction test.
        ENDAV(IK) = 0
        CUT(IK) = 1024
1352        READ(5,*) PART(IK), MCDET(IK), RECDET(IK)
      ELSEIF (NIT .EQ. -3) THEN
        DO 1362 IK = NELEM+1, NELEM+TOFELE
        EDGE(IK) = 1024  ! DEDXIT ignores edge correction test.
        ENDAV(IK) = 0
        CUT(IK) = 1024
1362        READ(5,*) MCDET(IK),RECDET(IK)
        DO 1364 JK = 1,NSTRATA
        IF(TOFELE .LE. 18) THEN
          READ(5,*) INF(JK),SUP(JK),(COMP(IK,JK),
     1                             IK=NELEM+1,NELEM+TOFELE)
        ELSE
          READ(5,*) INF(JK),SUP(JK),(COMP(IK,JK),
     1                                 IK=NELEM+1,NELEM+18)
          READ(5,*) (COMP(IK,JK),IK=NELEM+19,NELEM+TOFELE)
        ENDIF
1364        CONTINUE
      ENDIF
C
C
C --- Is monitor of ERD or RBS type?
      IF(EXTP.EQ.'ERD') THEN
            MDM=MCM
            RECPAR=RECMON
              IZDM=IZM
              IMON = IMON(1:Len_Trim(IMON))
      ELSE
            MDM=MP
            RECPAR=PROJ
              IZDM=IZP
              IMON = IMON(1:Len_Trim(IMON))
      ENDIF
C
C --- Convert projectile angle.
      RETA1=TETAIN/57.2957795d0
C
C --- convert monitor angle.
      RETA2M = RETA2M/57.2957795d0
C
C      PASN = PASN/1000.  ! Convert to mg/cm**2
      P1=PASN/SIN(RETA1)
      P2M=PASN/SIN(RETA2M)
C
C
C --- Write setup data to .REC file
      IF(NIT.EQ.-1) THEN
            WRITE (11,911) TARPRO
911            FORMAT(/10X,'MATRIX DEDUCED FROM ',
     1      'PROFILE ',A6,/10X,'DURING ONE ITERATION')
      ELSEIF(NIT.EQ.-2) THEN
            WRITE(11,910) TARPRO
910            FORMAT(/10X,' MATRIX DEDUCED FROM ',
     1      'COMPOSITION ',A6,/10X,'DURING ONE ITERATION',
     2      /10X,'THIS FILE HAS BEEN CONSTRUCTED USING A',
     3      ' KNOWN, UNIFORM COMPOSITION')
      ELSEIF(NIT.EQ.-3) THEN
            WRITE(11,913) TARPRO
913            FORMAT(/10X,' MATRIX DEDUCED FROM ',
     1      'STRATA ',A6,/10X,'DURING ONE ITERATION',
     2      /10X,'THIS FILE HAS BEEN CONSTRUCTED USING A',
     3      ' DEDUCED STRATUM STRUCTURE')
      ELSE
            WRITE(11,912) NIT
912            FORMAT(/10X,' RUN WITH ',I2,' ITERATIONS')
      ENDIF
C
C
      DO 400 I=1,1024  ! Clear array
      DO 400 J=1,20
400      IDECON(I,J)=0
C
C
C --- Write part of names of DEDX tables which are already known.
C             ITABLE(1,I) ==> Projectile entry table
C             ITABLE(2,I) ==> Monitor exit table
C             ITABLE(3,I) ==> Detector exit table
C             ITABLE(4,I) ==> Detector absorber table
C             ITABLE(5,I) ==> Monitor absorber table
      DO 100 I=1,20
          ZTABLE(1,1,I) = IZP
          MTABLE(1,I) = MP
          ITABLE(1,I)(1:2)=PROJ

          ZTABLE(1,2,I) = IZDM
          MTABLE(2,I) = MDM
100     ITABLE(2,I)(1:2) = RECPAR

        ZTABLE(1,5,1) = IZDM
        MTABLE(5,1) = MDM
      ITABLE(5,1)(1:2) = RECPAR
C
C --- Run monitor profile. Assume substrate made of monitor element.
C     Use array ( ,1) for initial profile.
        ZTABLE(2,1,1) = IZM
      ITABLE(1,1)(3:4) = RECMON ! Proj in.

        ZTABLE(2,2,1) = IZM
      ITABLE(2,1)(3:4) = RECMON ! Mon out.
C
        CALL SHOWTABLE
      CALL PROFILM (I,BSUP,BINF,DXABSM)
C
      IF(I.EQ.0) GO TO 23 ! failed
C
C
      ITT=0  ! Initialize iteration counter.
      OPEN(20, FILE='SURFENER.ERD')
      WRITE(20,*)'detected_atom  z  surface_energy (after absorber)'
C
C --- Run element profile for all NELEM elements.
      DO 2 IELE = 1,NELEM
      IF(TYPE(IELE) .EQ. 'ERD') THEN
        IDET(IELE) = IDER(IELE)(1:Len_Trim(IDER(IELE)))
        MDDET(IELE) = MCDET(IELE) ! Deduce masses and charge
        IZDD = IZDET(IELE)

          ZTABLE(1,3,IELE) = IZDD
          MTABLE(3,IELE) = MDDET(IELE)
        ITABLE(3,IELE)(1:2) = RECDET(IELE) ! Element in

          ZTABLE(1,4,IELE) = IZDD
          MTABLE(4,IELE) = MDDET(IELE)
        ITABLE(4,IELE)(1:2) = RECDET(IELE)  ! Element thru absorber
      ELSEIF(TYPE(IELE) .EQ. 'RBS') THEN
        IDET(IELE) = IDER(IELE)(1:Len_Trim(IDER(IELE)))
        MDDET(IELE) = MP
        IZDD = IZP

          ZTABLE(1,3,IELE) = IZP
          MTABLE(3,IELE) = MP
        ITABLE(3,IELE)(1:2) = PROJ  ! Projectile scatters

          ZTABLE(1,4,IELE) = IZP
          MTABLE(4,IELE) = MP
        ITABLE(4,IELE)(1:2) = PROJ  ! Projectile thru absorber
        ELSE
          write(11,*)'UNKNOWN EXPERIMENT TYPE ''',
     +                           TYPE(IELE),''' FOR IELE ',IELE
      ENDIF

      MCD = MCDET(IELE)
      MDD = MDDET(IELE)
      IZD = IZDET(IELE)

        ZTABLE(2,3,IELE) = IZM
      ITABLE(3,IELE)(3:4) = RECMON  ! through substrate
C
C --- NOTE: IZDD = IZD (ERD), IZP (RBS) , Used in ENERGY_CAL
C
        RETA2D(IELE) = RETA2D(IELE)/57.2957795d0
C --- Choose step size for detected element.
      P2D(IELE) = PASN/SIN(RETA2D(IELE))
C
C --- Deduce ratio of solid angles subtended by detectors used
C     to measure monitor & detected element.
C      IF(DETD(IELE) .EQ. DETM) THEN
C        ROMEGA(IELE) = 1.  ! Same detector ==> ratio is 1.
C      ELSE
C        IF(DETD(IELE) .EQ. 'P') THEN
C          ROMEGA(IELE) = RATIO ! Different ==> use ratio inputed.
C        ELSE
C          ROMEGA(IELE) = 1./RATIO ! Opposite ratio !
C        ENDIF
C      ENDIF

C --- From now, ROMEGAs are normalized to monitor (as if ROMEGAM = 1)
        ROMEGA(IELE) = ROMEGA(IELE)/ROMEGAM

C
C
C --- Fill in name of profile spectrum.
C      IPRO(IELE)(1:1) = 'P'
C      IPRO(IELE)(2:2) = TOFNUM
C      IPRO(IELE)(3:3) = PRONUM
C      IPRO(IELE)(4:5) = RECDET(IELE)
C      IPRO(IELE)(7:10) = '.SPC'
        IPRO(IELE) = '0'//IDER(IELE)(1:Len_Trim(IDER(IELE))-4)//'.PRO'

        CALL SHOWTABLE
C
      CALL PROFILD (I,IELE,FKD(IELE),SURFE)
      WRITE(20,*)RECDET(IELE),' ',IZDET(IELE),SURFE*1000.d0   ! surface energy in keV
C
      IF (I.EQ.0) GO TO 23 ! failed
C
2      CONTINUE  ! End of element profile loop
C
C
C --- Write data to .REC file
      WRITE(11,990) NELEM
990      FORMAT(/10X,'INITIAL PROFILING IS COMPLETE.',
     +      /10X,I2, ' ELEMENTS:')
C
C --- Flush log file buffered data to disk
        CALL Flush(11)
        CLOSE(20)

C
       DO 150 I=1,NELEM
      WRITE(11,991) RECDET(I)
991      FORMAT(20X,A2)
C
C --- Define full array of substrates being NELEM elements detected
C     in initial profiling

        ZTABLE(2,1,I) = IZDET(I)
      ITABLE(1,I)(3:4)=RECDET(I) ! Proj in

        ZTABLE(2,3,I) = IZDET(I)
      ITABLE(3,I)(3:4)=RECDET(I) ! Element out

        ZTABLE(2,2,I) = IZDET(I)
150      ITABLE(2,I)(3:4)=RECDET(I) ! Monitor out
        CALL SHOWTABLE
C
C --- If no iterations requested, end program
      IF(NIT.EQ.0) GO TO 22
C
C --- Start iteration procedure.
11      IF(NIT .GE. 1) CALL EDGE_FIND ! Routine to deduce upper edge of
C                      profile and average atomic ratio for DEDXIT.
C
      IELE=1  !  Begin with first profiled element
      MCD=MCDET(IELE) ! Deduce masses & charges
      IZD=IZDET(IELE)
      MDD=MDDET(IELE)
C
C      IF(TYPE(IELE) .EQ. 'ERD') THEN
C        IDET(3:4) = RECDET(IELE) ! Deduce energy spectrum
C        IDET(5:) = '.SPC'
C      ELSEIF(TYPE(IELE) .EQ. 'RBS') THEN
C        IDET(3:4) = PROJ
C        IDET(5:) = RECDET(IELE) // '.SPC'
C      ENDIF
C
      ITT=ITT+1  ! Iteration counter
C
      WRITE(11,992) ITT
      WRITE(6,992) ITT
992      FORMAT(/10X,'ITERATION ',I1,' STARTING')
C
C --- Data deduced between this point and statement 98 need only
C     be done once, i.e. on the first iteration.
      IF(ITT.GT.1) GO TO 98
C
C --- The iteration subroutines use the subroutine DEDXIT to deduce
C     the energy losses. This routine deduces a composite DEDX table
C     based on the substrate composition deduced either from an old
C     profile, a fixed composition, or the results of the previous
C     iteration (or the simple profile if ITT=0) of the present run.
C
C --- If NIT = -1, use an old profile to deduce substrate composition
C     for one iteration
      IF(NIT.EQ.-1) THEN
C
C --- Read in profile spectra into part of array beyond storage of
C     actual detected elements NELEM. TOFELE is number of elements
C     in old profile.
        DO 700 JJ=NELEM+1,NELEM+TOFELE
        CALL DISQUE(IPRO(JJ),IDECON(1,JJ),1,ISUC,0)
C
         IF (ISUC.NE.1) THEN
          WRITE(11,755)
          WRITE(11,797)
755          FORMAT(/10X,'PROFILE UNAVAILABLE;')
797          FORMAT(/10X,' ABANDONING ITERATION PROCEDURE')
          GO TO 23
        ENDIF
C
C --- Similarly, use array space beyond NELEM to store DEDX tables
C     for the elements from the old profile. First, store array of
C     elements composing substrate for detected elements
          ZTABLE(2,3,JJ) = IZELEM(IPRO(JJ)(4:5))
        ITABLE(3,JJ)(3:4)=IPRO(JJ)(4:5)
        write(*,*) 'this need to be fixed so the program has another',
     1 ' way to know the element corresponding to the profile provided'
C
        DO 701 II=1,2
C
C --- and do the same for monitor and projectile & read in for monitor
C     and projectile. Tables for detected elements will be read in later.
          ZTABLE(2,II,JJ) = IZELEM(IPRO(JJ)(4:5))
        ITABLE(II,JJ)(3:4)=IPRO(JJ)(4:5)
        write(*,*) 'this need to be fixed so the program has another',
     1 ' way to know the element corresponding to the profile provided'
C
701        CONTINUE
700        CONTINUE
        CALL SHOWTABLE
C
C --- If NIT = -2, use a known, fixed composition for substrate (in
C     the same manner as for NIT = -1.
      ELSEIF(NIT.EQ.-2) THEN
        DO 860 JJ=NELEM+1,NELEM+TOFELE
        DO 863 MM=1,1024  ! Across entire spectrum
863        IDECON(MM,JJ)=PART(JJ)*10000.d0  ! Normalize to 10,000
C
C --- Use array space beyond NELEM to store DEDX tables for the
C     elements in the fixed composition. First, store array of
C     elements composing substrate for detected elements.
          ZTABLE(2,3,JJ) = IZDET(JJ)
        ITABLE(3,JJ)(3:4)=RECDET(JJ)
C
C --- and do the same for monitor and projectile & read in for monitor
C     and projectile. Tables for detected elements will be read in later.
        DO 864 II=1,2
          ZTABLE(2,II,JJ) = IZDET(JJ)
        ITABLE(II,JJ)(3:4)=RECDET(JJ)
864        CONTINUE
860        CONTINUE
        CALL SHOWTABLE
C
C --- If NIT = -3, use a deduce, strata composition for substrate (in
C     the same manner as for NIT = -1.
      ELSEIF(NIT.EQ.-3) THEN
        DO 870 JJ=NELEM+1,NELEM+TOFELE
        DO 873 KK = 1,NSTRATA
        DO 873 MM = INF(KK),SUP(KK)
873        IDECON(MM,JJ) = COMP(JJ,KK)*10000.d0  ! Normalize to 10,000
C
C --- Calculate IDECON between 2 strata by drawing straight line.
        DO 875 KK = 1,NSTRATA-1
        XM = (COMP(JJ,KK+1) - COMP(JJ,KK)) / (INF(KK+1) - SUP(KK))
        XINT = COMP(JJ,KK) - SUP(KK) * XM
        DO 875 MM = SUP(KK)+1,INF(KK+1)-1
875        IDECON(MM,JJ) = (MM * XM + XINT) * 10000.d0
C
C --- Use array space beyond NELEM to store DEDX tables for the
C     elements in the fixed composition. First, store array of
C     elements composing substrate for detected elements.
          ZTABLE(2,3,JJ) = IZDET(JJ)
        ITABLE(3,JJ)(3:4)=RECDET(JJ)
C
C --- and do the same for monitor and projectile & read in for monitor
C     and projectile. Tables for detected elements will be read in later.
        DO 874 II=1,2
          ZTABLE(2,II,JJ) = IZDET(JJ)
        ITABLE(II,JJ)(3:4)=RECDET(JJ)
874        CONTINUE
870        CONTINUE
        CALL SHOWTABLE
C
C --- If NIT is greater than zero, execute iteration using elements
C     deduced in original profiling to deduce the substrate composition.
C     Read in tables for monitor & projectile.
      ELSEIF(NIT.GE.1) THEN
        TOFELE=0
       ELSE
        GO TO 23
      ENDIF
C
98      CALL ITERAM(ISUC,P1) ! Run monitor iteration procedure
      IF (ISUC.EQ.0) GO TO 23
      SIGMIT = SIG(MP,MCM,MDM,IZP,IZM,RETA1,RETA2M,ERMONIT)
C
5      IF(TOFELE.NE.0) THEN   ! that is, NIT = -1, -2 or -3
C
C --- Read in DEDX tables for detected element.
        DO 757 JJ=NELEM+1,NELEM+TOFELE
C
        IF(TYPE(IELE) .EQ. 'ERD') THEN
            ZTABLE(1,3,JJ) = IZDET(IELE)
            MTABLE(3,JJ) = MDDET(IELE)
          ITABLE(3,JJ)(1:2)=RECDET(IELE)
        ELSEIF(TYPE(IELE) .EQ. 'RBS') THEN
            ZTABLE(1,3,JJ) = IZP
            MTABLE(3,JJ) = MP
          ITABLE(3,JJ)(1:2) = PROJ
        ENDIF
C
757        CONTINUE
        CALL SHOWTABLE
C
      ELSE  ! Regular iteration procedure
C
C --- Read in DEDX tables.
        DO 200 I=1,NELEM
C
        IF(TYPE(IELE) .EQ. 'ERD') THEN
            ZTABLE(1,3,I) = IZDET(IELE)
            MTABLE(3,I) = MDDET(IELE)
          ITABLE(3,I)(1:2)=RECDET(IELE)
        ELSEIF(TYPE(IELE) .EQ. 'RBS') THEN
            ZTABLE(1,3,I) = IZP
            MTABLE(3,I) = MP
          ITABLE(3,I)(1:2) = PROJ
        ENDIF
C
200        CONTINUE
        CALL SHOWTABLE
      ENDIF
C
C --- Perform element iteration procedure
      CALL ITERAD(ISUC,IELE,FKD(IELE),ITT,NIT,SIGMIT,RATIO)
C
C
      IF(IELE.EQ.NELEM) GOTO 4  ! Element list finished !
      IELE=IELE+1  ! Next element
      MCD=MCDET(IELE)  ! Deduce masses & charges for next element
      IZD=IZDET(IELE)
      MDD=MDDET(IELE)
C
C       report percentage of work done
        CLOSE(3)
        OPEN(3, FILE='PROGRESS.ERD')
        WRITE(3,*)((ITT-1)*NELEM+IELE)*100/(NELEM*NIT)
        CLOSE(3)
C
C
C      IF(TYPE(IELE) .EQ. 'ERD') THEN
C        IDET(3:4)=RECDET(IELE) ! Deduce next energy spectrum
C        IDET(5:) = '.SPC'
C      ELSEIF(TYPE(IELE) .EQ. 'RBS') THEN
C        IDET(3:4) = PROJ
C        IDET(5:) = RECDET(IELE) // '.SPC'
C      ENDIF
C
      GOTO 5  ! Iterative profile of next element
C
C --- For regular iteration procedure, the results of this iteration
C     are read into profile array to be used by next iteration.
C     For NIT = -1, -2 or -3, array is read in for final calculation of
C     edge channel and average.
4      DO 857 KL = 1, NELEM
      IF(NIT .GE. 0) THEN
        WRITE(IPRO(KL)(1:1),6000) ITT ! Iteration # is first digit
      ELSEIF(NIT .EQ. -1) THEN
        IPRO(KL)(1:1) = 'P'   ! Only iteration
      ELSEIF(NIT .EQ. -2) THEN
        IPRO(KL)(1:1) = 'C'   ! Only iteration
      ELSEIF(NIT .EQ. -3) THEN
        IPRO(KL)(1:1) = 'S'   ! Only iteration
      ENDIF
6000      FORMAT(I1)
857      CALL DISQUE(IPRO(KL),IDECON(1,KL),1,ISUC,0)
C
C --- For NIT = -1, -2 or -3, only one iteration is necessary. Exit program
      IF(ITT.EQ.NIT .OR. TOFELE .NE. 0) GO TO 22
C
      GO TO 11 ! Start next iteration !
C
C
C --- Normal exit
22      CLOSE(3)
        OPEN(3, FILE='PROGRESS.ERD')
        WRITE(3,*) 101
        CLOSE(3)

        CALL EXIT

23      write(6,*) 'RUN STOPPED'

        CLOSE(3)
        OPEN(3, FILE='PROGRESS.ERD')
        WRITE(3,*) -1
        CLOSE(3)


C
      END

C
C----------------------------------------------------------------------
      SUBROUTINE ABONDE (R,ERD,EOD,BUFF,DET,ED,BD,RETA1,RETA2,IELE)
C----------------------------------------------------------------------
C
C  PURPOSE:  to convert the energy spectrum to profile spectrum
C            once the energy to depth relation is known
C
C  R       -> normalization constant
C  ERD     -> table of projectile energies as a function of depth
C  EOD     -> table of scatttered energies as a function of depth
C  BUFF    -> profile spectrum
C  DET     -> energy spectrum
C  ED      -> slope of energy/channel
C  BD      -> intercept
C  RETA1   -> entry angle [rad]
C  RETA2   -> exit angle [rad]
C  IELE    -> element number
C -------------------------------------------------------------------------
C              *COMMON BLOCKS*
C  /D/
C      MP......... mass of projectile [amu]
C      MCM........ mass of monitor [amu]
C      MDM........ mass of detected monitor (MCM if ERD, MP if RBS)
C      MCD........ mass of target element [amu]
C      MDD........ mass of detected element (MP if RBS, MCD if ERD)
C      IZP........ Z of projectile
C      IZM........ Z of monitor
C       IZDM....... Z of detected monitor (IZP if RBS, IZM if ERD)
C      IZD........ Z of target element
C      IZDD....... Z of detected element (IZP if RBS, IZD if ERD)
C
C  /CAL/
C       ECAL(1024,30).... Energy calibration for element(I)
C                         The value of ECAL is the UPPER edge of the
C                         channel
C       ECALM(1024)...... Energy calibration of monitor
C       ICHN_SURF(30).... Surface channel for element (I)
C      ARHOD(30)........ Constant for PHD calculation
C      ARHOM............ Constant for PHD calculation for monitor
C       ALPHA_SLOPE...... Slope of alpha particle energy calibration
C -------------------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      DOUBLE PRECISION  ERD(500),EOD(500),ECAL(1024,30),ECALM(1024)
      DIMENSION ICHN_SURF(30)
      DOUBLE PRECISION  MP,MCM,MDM,MCD,MDD,ARHOD(30)
      INTEGER*4 BUFF(1024),DET(1024)
      COMMON/D/MP,MCM,MDM,MCD,MDD,IZP,IZM,IZDM,IZD,IZDD
      COMMON/CAL/ECAL,ECALM,ICHN_SURF,ARHOD,ARHOM,ALPHA_SLOPE
C
C
C      ***** CALCUL DE 5 POINTS AVANT LA SURFACE *****
C
      GAP=EOD(1)-EOD(2)
      EAV = ERD(1)
      SIGD = SIG(MP,MCD,MDD,IZP,IZD,RETA1,RETA2,EAV)
C
      DO 35 I=1,5
      ICHN1 = ICHN_SURF(IELE) + 10  ! Starting channel for SURF1
C
C --- ARHOD > 0 --> Energy calibration calculated from energy dependence
C                   of PHD and stored in array ECAL(1024,IELE)
C --- ARHOD = 0 --> Use old calibration [E=ED*(CHN-1) +BD]
C
      IF (ARHOD(IELE).GT.0) THEN
        ABON = R * SURF1(EOD(1)+I*GAP,EOD(1)+(I-1)*GAP,DET,
     1                 ECAL(1,IELE),ICHN1)/SIGD
      ELSE
        ABON = R * SURF(EOD(1)+I*GAP,EOD(1)+(I-1)*GAP,DET,
     1                 ED,BD)/SIGD
      ENDIF
      BUFF(6-I)=10000.d0*ABON+0.5d0
35      CONTINUE

C
C      ***** CALCUL DES CONCENTRATIONS RELATIVES *****
C
      ICHN1 = ICHN_SURF(IELE) + 1  ! Starting channel for SURF1
      I=1
40      J=I+1
C
C --- Do not calculate below energy defect.
      IF(EOD(J).GT.0.0.AND.EOD(J).GT.BD) THEN
            EAV = (ERD(I)+ERD(J))/2.d0
            SIGD = SIG(MP,MCD,MDD,IZP,IZD,RETA1,RETA2,EAV)
              IF (ARHOD(IELE).GT.0) THEN
              ABON=R*SURF1(EOD(I),EOD(J),DET,ECAL(1,IELE),
     1                      ICHN1)/SIGD
              ELSE
              ABON=R*SURF(EOD(I),EOD(J),DET,ED,BD)/SIGD
              ENDIF

            BUFF(I+5)=10000.d0*ABON+0.5d0
            I=J
            GO TO 40
      ELSE
            DO 50 K=I+5,1024
50            BUFF(K)=0
            RETURN
      ENDIF
      END
C
C
C
C
C-----------------------------------------------------------------
      FUNCTION CINEM(MP,MC,MD,RETA1,RETA2)
C-----------------------------------------------------------------
C
C  PURPOSE:  calculate the kinematic factor of recoiled atom
C
C  MP     -> mass of projectile (amu)
C  MC     -> mass of target element (amu)
C  MD     -> mass of detected element (amu)
C  RETA1  -> angle of projectile (radians)
C  RETA2  -> angle of detected element (radians)
C-----------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION  MP,MC,MD
      IF(ABS(MP-MD).LT.0.1) THEN
C
C      **** CAS  RBS ****
C
            C1=MP/(MP+MC)
            C2=MC/MP
            CINEM=C1*C1*(COS(RETA1+RETA2)+SQRT(C2*C2
     +            -(SIN(RETA1+RETA2))**2))**2
      ELSE
C
C      **** CAS ERD ****
C
         CINEM=(4.0d0*MP*MC/(MP+MC)**2)*(COS(RETA1+RETA2))**2
      ENDIF
      RETURN
      END
C
C
C
C
C------------------------------------------------------------------
      FUNCTION DEDX(E1,J,K)

C FrSch: pour les pouvoirs d'arret ZG95, TABLE() contient les parametres
C de NBRKS polynomes d'ordre 4 dont l'intervalle de validite en
C energie est entre TABLE(I,...) et TABLE(I+6,...) et les coefficients
C du polynome sont TABLE(I+1,...) jusqu'a TABLE(I+5,...)
C

C------------------------------------------------------------------
C  PURPOSE:  calculate the DEDX value
C
C  E1  -> energy
C  J   -> what DEDX item wanted
C  K   -> element number
C -----------------------------------------------------------------
C              *COMMON BLOCKS*
C  /TADEDX/
C      ITABLE(5,30). table containing names of DEDX tables [*4]
C                     (1,I) : projectile entry thru element(I)
C                     (2,I) : detected monitor exit thru element(I)
C                     (3,I) : detected element exit thru element(I)
C                     (4,I) : detected element thru absorber
C                     (5,I) : detected monitor thru absorber
C      ZTABLE(2,5,30). table containing Z1, Z2 correspoinding to ITABLE elements
C      MTABLE(5,30).   table containing M1 correspoinding to Z1 in ITABLE elements
C
C  /A/
C      TABLE(60,5,30).. DEDX tables corresponding to ITABLE(5,30)
C -----------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      CHARACTER*30 TEMPS
      CHARACTER*4 ITABLE
        INTEGER ZTABLE, Z1, Z2
        DOUBLE PRECISION  M1, EINPUT, MTABLE

      DIMENSION ITABLE(5,30), ZTABLE(2,5,30), MTABLE(5,30)

      COMMON/TADEDX/ITABLE, ZTABLE, MTABLE  !!,ITAB

      IF(E1.LT.0.0) THEN
                CALL CTIME(TIME8(), TEMPS)
            WRITE(11,100) TEMPS,E1,J,K
100      FORMAT(/10X,A30,/10X,'ENERGY= ',F6.3,' INPUT TO DEDX',
     +   ' FOR ITABLE(',I1,',',I1,') IS NEGATIVE. EXECUTION STOPPED.')
                CLOSE(3)
                OPEN(3, FILE='PROGRESS.ERD')
                WRITE(3,*) -1
                CLOSE(3)

            CALL EXIT
      ELSE

          Z1 = ZTABLE(1,J,K)
          M1 = MTABLE(J,K)  ! 0 for most abundant isotope

C     (no more) special cases
C          IF(Z1.EQ.-1) THEN ! this is element HY, that is deuterium
C            Z1=1
C            M1=2.002
C          ELSEIF(Z1.EQ.-1) THEN ! this is element HY, that is deuterium
C            Z1=1
C            M1=2.002
C          ENDIF

          Z2 = ZTABLE(2,J,K)


          EINPUT = E1*1000.d0

          DEDX = STOP96(Z1,M1,Z2,EINPUT)/1000.d0 ! in MeV/(DPUN)

C        write(11,*)'dedx: Z1,M1,Z2,Ein,dE/dx:',Z1,M1,Z2,EINPUT,DEDX

      ENDIF
      RETURN
      END
C
C
C
C
C
C----------------------------------------------------------------
      FUNCTION DEDXIT(E,ITA,N)
C----------------------------------------------------------------
C
C  PURPOSE: calculate the DEDX values using Bragg's Law.
C
C  E     -> energy of particle in layer
C  ITA   -> table type  1=proj   2=mon   3=det
C  N     -> channel number of IDECON(N,IELE) ie. depth
C----------------------------------------------------------------
C              *COMMON BLOCKS*
C  /IDE/
C      IDECON(1024,30).. profile spectra for target elements
C      MCDET(30)........ mass of target elements [amu]
C
C  /D/
C      MP......... mass of projectile [amu]
C      MCM........ mass of monitor [amu]
C      MDM........ mass of detected monitor (MCM if ERD, MP if RBS)
C      MCD........ mass of target element [amu]
C      MDD........ mass of detected element (MP if RBS, MCD if ERD)
C      IZP........ Z of projectile
C      IZM........ Z of monitor
C       IZDM....... Z of detected monitor (IZP if RBS, IZM if ERD)
C      IZD........ Z of target element
C      IZDD....... Z of detected element (IZP if RBS, IZD if ERD)
C
C  /EL/
C       NELEM....... number of elements
C      TOFELE...... number of elements in specified target profile
C
C  /EDGE/
C      EDGE(30).... depth limit of profile for element(I) [ug/cm2]
C      ENDAV(30)... calculated average of element with depth > EDGE
C      CUT(30)..... predefined depth limit of element(I) [ug/cm2]
C---------------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        
      INTEGER*4 N,IDECON(1024,30),TOFELE,EDGE(30),ENDAV(30),CUT(30)
      DOUBLE PRECISION  MCDET(30),MP,MCM,MDM,MDD,MCD
C
      COMMON/IDE/IDECON,MCDET
      COMMON/EL/NELEM,TOFELE
      COMMON/D/MP,MCM,MDM,MCD,MDD,IZP,IZM,IZDM,IZD,IZDD
      COMMON/EDGE/EDGE,ENDAV,CUT
C
      IF(TOFELE.NE.0) THEN ! Use old profile or fixed composition.
            KK=NELEM+1
            LL=NELEM+TOFELE
      ELSE                 ! Use profiles from last iteration.
            KK=1
            LL=NELEM
      ENDIF
C
      FM=0.d0
      DEDXIT=0.d0
      DO 10 J=KK,LL
C
C --- If channel N for element J is below cutoff (due to energy window),
C     then do normal calculation, otherwise, use the average deduced
C     from the last few channels before the cutoff as the abundance so
C     that the iteration procedure does not produce a spurious rise in
C     concentration in the other elements.
C
      IF(N .GE. CUT(J)) THEN   ! Cutoff procedure
        ABONI=0.d0
      ELSEIF(EDGE(J) .GT. N) THEN  ! Normal procedure
        ABONI=(IDECON(N,J)/10000.d0)*MCDET(J)
      ELSE                     ! Extension procedure
        ABONI=(ENDAV(J)/10000.d0)*MCDET(J)
      ENDIF
C
C        write(97,*) 'N, CUT(J), EDGE(J), ENDAV(J), J, FM, ABONI, ITA, E,
C     +   DEDXIT, DEDX(E,ITA,J)'
C        write(97,*)  N, CUT(J), EDGE(J), ENDAV(J), J, FM, ABONI, ITA, E,
C     +   DEDXIT, DEDX(E,ITA,J)
      FM=FM + ABONI
10      DEDXIT=DEDXIT + ABONI*DEDX(E,ITA,J)
C
      IF(FM.EQ.0.0) THEN     ! Look for RECDET same as RECMON
        DO 100 K = KK,LL     ! to use as DEDX value.
        IF(MCDET(K) .EQ. MCM) THEN
          DEDXIT = DEDX(E,ITA,K)
          RETURN
        ENDIF
100        CONTINUE
C
        DEDXIT = DEDX(E,ITA,1) ! Use first element if RECMON not found.
C
      ELSE  ! Return value from Bragg's Law.
        DEDXIT=DEDXIT/FM
      ENDIF
C
      RETURN
      END
C
C
C
C
C-------------------------------------------------------------
      SUBROUTINE DISQUE(ISPC,BUFF,IFUNC,ISUC,IX)
C-------------------------------------------------------------
C
C      IFUNC=1  READ        =2  WRITE
C      ISUC=1   SUCCESS     =0  ERROR
C       IX=1 use PASN for X  =0  use I for X   (for writing)
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER*4 BUFF(1024)
      CHARACTER*(*) ISPC
        character*1024 WORKDIR
        character*1024 filename

      COMMON/DSK/WORKDIR,PASN2

      filename = WORKDIR(1:Len_Trim(WORKDIR))//ISPC(1:Len_Trim(ISPC))
C
      CLOSE(12)    ! Insure UNIT 12 is closed.
C
C      *** LECTURE ****
C
      IF(IFUNC.EQ.1) THEN
        OPEN(12,FILE=filename,STATUS='OLD',ERR=199) !,
C     1 FORM='UNFORMATTED')
        DO 10 I=1,1024
        READ(12, *, END=299) XX, BUFF(I)   ! first column is ignored
10        CONTINUE
      ELSEIF(IFUNC.EQ.2) THEN
C
C      **** ECRITURE ***
C
      OPEN(12,FILE=filename)   !,STATUS='NEW') !,FORM='UNFORMATTED')
        DO 20 I=1,1024
            IF(IX.EQ.1) THEN
              WRITE(12, *) DBLE(I-6)*PASN2, BUFF(I) ! write depth scale in first column
            ELSE
              WRITE(12, *) I, BUFF(I) ! write I in first column
            ENDIF
20        CONTINUE
      ENDIF
      CLOSE(12)
      ISUC=1
      RETURN
C
C
199      WRITE(11,*)'FILE ', filename,' DOES NOT EXIST'
        WRITE(11,120) ISPC
120      FORMAT(/10X,'FILE ',A22,' DOES NOT EXIST')
      ISUC=0
      RETURN
299      WRITE(11,*)'FILE ', filename,' IS INCOMPLETE OR EMPTY'
C220      FORMAT(/10X,'FILE ',A1000,' IS INCOMPLETE OR EMPTY')
      CLOSE(12)
      ISUC=0
      RETURN
      END
C
C
C
C--------------------------------------------------------------------------
      SUBROUTINE EDGE_FIND
C--------------------------------------------------------------------------
C      This routine finds the upper cutoff channel of each profile (EDGE)
C      and deduces the average of the last few channels (ENDAV). This is
C      used to avoid the spurious change in concentration in other elements
C      beyond this point due to the apparent absence of said profile.
C--------------------------------------------------------------------------
C              *COMMON BLOCKS*
C  /C/
C      RETA1...... angle of incidence (theta-in) [radians]
C      RETA2M..... monitor detector angle [radians]
C      RETA2D(30). exit angle of detected element [radians]
C      P1......... step size (PASN) corrected for RETA1 [ug/cm2]
C      P2M........ step size corrected for RETA2M [ug/cm2]
C      P2D(30).... step size corrected for RETA2D [ug/cm2]
C      PASN....... step size [ug/cm2]
C      EIN........ energy of projectile [MeV]
C      EFIN....... lower energy limit [MeV]
C
C  /IDE/ 
C      IDECON(1024,30).. profile spectra for target elements
C      MCDET(30)........ mass of target elements [amu]
C
C  /EL/
C       NELEM....... number of elements
C      TOFELE...... number of elements in specified target profile
C
C  /EDGE/
C      EDGE(30).... depth limit of profile for element(I) [ug/cm2]
C      ENDAV(30)... calculated average of element with depth > EDGE
C      CUT(30)..... predefined depth limit of element(I) [ug/cm2]
C--------------------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      INTEGER*4 EDGE(30),ENDAV(30),IDECON(1024,30),TOFELE,CUT(30)
      DOUBLE PRECISION  MCDET(30)
      DIMENSION RETA2D(30),P2D(30)
C
      COMMON/EDGE/EDGE,ENDAV,CUT
      COMMON/IDE/IDECON,MCDET
      COMMON/EL/NELEM,TOFELE
      COMMON/C/RETA1,RETA2M,RETA2D,P1,P2M,P2D,PASN,EIN,EFIN
C
C ***********************************************************************
C     SPECIAL NOTE: In truth, these limits should depend on the
C                   statistics in the energy spectra. Thus, the limits
C                   may be related to the statistics in the monitor
C                   window and the atomic ratios. It will be done later!
C                   Presently, we deduce from SQRT(IDECON).
      SC1 = 1.00d0  ! Lower scatter limit for edge search
      SC2 = 1.00d0  ! Upper scatter limit for edge search
      SC3 = 0.98d0  ! Lower scatter limit for range search
      SC4 = 1.02d0  ! Upper scatter limit for range search
C ***********************************************************************
C
      DO 1 K = 1,NELEM
C
C --- No need to deduce EDGE for secondary detector of ORTEC chamber.
C      IF(DETD(K) .EQ. 'S' .AND. CHAM .EQ. 'ORT') THEN
C        EDGE(K) = 1024
C        ENDAV(K) = 0
C        GO TO 1
C      ENDIF
C
      L = 600/PASN ! *1000.)  ! Count down starting at channel 600
C
2      L = L - 1
      IF(IDECON(L,K) .EQ. 0) GO TO 2  ! Look for non-zero channel
      LL = L - 5   ! Absolute limit for search for weak layers
      LM = L - 8   ! Absolute limit for all layers, assume zero
C
      AV1 = 0
      AV2 = 0
      DO 3 J = L-1,L  ! Sum contents of 2 channels of current window.
3      AV1 = AV1 + DBLE(IDECON(J,K))
C
4      AV2 = AV1 - DBLE(IDECON(L,K) + IDECON(L-2,K))  ! Window down 1 channel
C
      IF(AV2 .EQ. 0) THEN  ! If zero, then clearly not an energy
        EDGE(K) = 1024     ! cutoff. Skip out to prevent divide check
        ENDAV(K) = 0
        GO TO 1
      ENDIF
C
      TEST = AV2/AV1 ! Test that averages are roughly the same
C
C***      XIDE = SQRT(FLOAT(IDECON(L,K))) / 2. ! TEST at 2% for 10,000.
C***      IF(XIDE .LE. 20.) XIDE = 20.  ! Limit to 5%.
      XIDE = 20.d0  ! Limit to 5%.
      IF(TEST .LE. (1.d0 - SC1/XIDE) .OR.
     1  TEST .GT. (1.d0 + SC2/XIDE)) THEN ! Not a plateau
C
        IF(L .LE. LL .AND. AV2 .LE. 1000.) THEN ! Low conc.> wild scatter
          L = LL + 3   ! Choose 2 channels below edge arbitrarily
          EDGE(K) = L
          ENDAV(K) = IDECON(L,K)  ! Take average from 1 channel.
          GO TO 1
        ENDIF
C
        IF(L .LE. LM) THEN    ! After 8 channels, assume that
          EDGE(K) = 1024      ! edge is the actual end of the layer
          ENDAV(K) = 0        ! and not an energy cutoff.
          GO TO 1
        ENDIF
C
        L = L - 1   ! Go down a channel and look for plateau again
        AV1 = AV2
        GO TO 4
      ENDIF
C
C --- Now upper channel of plateau has been found. Routine now looks for
C     width of plateau (to a maximum of 10 channels) to use for averaging.
C
      EDGE(K) = L
      M = 2   ! Initial width of window
      AV1 = AV1/M   ! Average concentration of  M channel window
6      AV2 = (M*AV1 + DBLE(IDECON(L-M,K))) / (M+1) ! Average of M+1 window
      TEST = AV2/AV1  ! Test for start of new layer
C
      IF(TEST .GT. SC3 .AND. TEST .LT. SC4 .AND. M .LT. 10) THEN
        M  = M + 1  ! Window may be made wider
        AV1 = AV2   ! New window becomes old window
        GO TO 6
      ENDIF
C
      ENDAV(K) = AV1
1      CUT(K) = 1024
      RETURN
      END
C
C
C
C
C--------------------------------------------------------------------------
      SUBROUTINE ENERGY_CAL(ALPHA_SLOPE,IELE,IZD,MDD,SLOPE,PHD,FKD,
     1                     DXABS,JTAB,ARHO,ECAL,ICHAN)
C--------------------------------------------------------------------------
C
C --- Calculate energy calibration from energy dependence of PHD for a
C     given element. The alpha calibration slope is estimated by the
C     slope of a low Z surface peak (CX or OX).
C     The channel corresponding to the surface energy (less absorber loss)
C     is calculated from the slope and intercept (surface PHD) determined
C     in CALIBRATE. (This essentially gives the original surface chn which
C     was used to calculate the slope in the first place, but allows for
C     minimal change to the original user calibration interface.)
C
C     The PHD calculation follows:
C     "A calibration procedure for the response of SI surface-barrier
C     detectors to heavy ions", S.B. Kaufman et al., NIM 115(1974) 47-55
C
C
C     For Z<4, the PHD calculation used here is not appropriate.
C     In this case, the old calibration is used
C     i.e.   E = ED*[(I-1)] + BD
C
C *** The low Z elements are flagged by ARHO = 0.0
C
C --- Energy calibration is in MeV ---
C
C   ALPHA_SLOPE -> Slope of alpha particle energy calibration
C   IELE        -> Detected element number (1 for monitor)
C   IZD         -> Z of element
C   MDD         -> mass of element
C   SLOPE       -> calculated energy slope for element
C   PHD         -> PHD (intercept)  for surface energy
C   FKD         -> Kinematic factor
C   DXABS       -> thickness of monitor absorber
C   JTAB        -> DEDX table number (4=detected element, 5=monitor)
C   ARHO        -> detector resistivity constant for PHD calculation
C   ECAL        -> array of energy calibration. The energy value is that of
C                  the UPPER edge of the channel bin.
C   ICHAN        -> channel corresponding to surface (used in SURF)
C
C -------------------------------------------------------------------------
C              *COMMON BLOCKS*
C  /C/
C      RETA1...... angle of incidence (theta-in) [radians]
C      RETA2M..... monitor detector angle [radians]
C      RETA2D(30). exit angle of detected element [radians]
C      P1......... step size (PASN) corrected for RETA1 [ug/cm2]
C      P2M........ step size corrected for RETA2M [ug/cm2]
C      P2D(30).... step size corrected for RETA2D [ug/cm2]
C      PASN....... step size [ug/cm2]
C      EIN........ energy of projectile [MeV]
C      EFIN....... lower energy limit [MeV]
C
C  /TADEDX/
C      ITABLE(5,30). table containing names of DEDX tables [*4]
C                     (1,I) : projectile entry thru element(I)
C                     (2,I) : detected monitor exit thru element(I)
C                     (3,I) : detected element exit thru element(I)
C                     (4,I) : detected element thru absorber
C                     (5,I) : detected monitor thru absorber
C      ZTABLE(2,5,30). table containing Z1, Z2 correspoinding to ITABLE elements
C      MTABLE(5,30).   table containing M1 correspoinding to Z1 in ITABLE elements
C
C -------------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      DOUBLE PRECISION  MDD,K_LSS,M2
      DIMENSION ECAL(1024),RETA2D(30),P2D(30)
      COMMON/C/RETA1,RETA2M,RETA2D,P1,P2M,P2D,PASN,EIN,EFIN
C
      DATA EL_CHG2/1.4398E-13/  ! e^2 in Mev-cm
C
C --- Calculate chn corresponding to surface energy (less absorber loss)
C     The slope was calculated in CALIBRATE from:
C                         ESURF = ((CHN-1)+.5)*SLOPE + PHD  (MeV)
C
C     N.B. (CHN-1) gives E_alpha=0 at chn 1
C          (+.5)  Puts the surface energy at the upper edge of the channel
C
        write(11,*) 'going into energy_cal'
      DO 100 I=1,1024
100      ECAL(I) = 0.0d0
      ICHAN = 0
C
C --- Check for low Z (<4) elements
      IF (ARHO .EQ. 0) RETURN   ! Use old calibration format
C
C --- Energy at surface
      E0 = EIN * FKD
        write(11,*) 'ein=',ein,' fkd=',fkd,' dxabs=',dxabs
C
C --- Correct for absorber
        write(11,*) 'going to correct for absorber'
      IF(DXABS.EQ.0.0) THEN  ! Correct for monitor absorber.
            ESURF = E0
      ELSE
CC now allways the case        IF(ITAB(1:1) .EQ. 'Z') THEN  ! Zieg uses DEDX
              NABS = 1000
            EX = E0
                write(11,*) 'ex=',ex,' nabs=',nabs
                write(11,*) 'ex  jtab  iele dedx'
            DO 50 KK = 1,NABS
                IF(MOD(KK,NABS/10).EQ.0) then
                    write(11,*) KK, ex,jtab, IELE, DEDX(EX,JTAB,IELE)
                    CALL Flush(11)
                ENDIF
50            EX = EX - DEDX(EX,JTAB,IELE)*DXABS/1000.d0
            ESURF = EX
            write(11,*) 'E after absorber:', ex,jtab,iele
CC        ENDIF
      ENDIF
C
      EEDGE = ESURF - PHD
      CHAN = EEDGE/SLOPE + .5d0
C      WRITE(99,*) EIN,FKD,E0,ESURF,SLOPE,PHD,EEDGE,CHAN
      ICHAN = NINT(CHAN)
C
C --- Calculate energy calibration
C
C --- Define Si detector mass, charge
      IZ2 = 14
      M2 = 28.0680 d0
C
C --- Define LSS proportionality factor (E_LSS = K_LSS * E(Mev))
C
      K_LSS = TH_FERMI(IZD,IZ2)*M2/(IZD*IZ2*EL_CHG2*(MDD+M2))
C
C --- Define surface channel energy
C     Note that the value of ECAL is the energy of the UPPER edge of the
C     channel bin!!
C
      ECAL(ICHAN) = ESURF
C      WRITE(6,*) ICHAN,ECAL(ICHAN),SLOPE*(ICHAN-.5)+PHD
C
C --- Calculate remaining channels from the relation:
C                   dE/dCHN = Alpha_slope/(1 - dPHD/dE)
C
C      WRITE(99,*) 'Z1=', IZD, ' Z2=', IZ2
      I1 = ICHAN-1
      DO 140 I=I1,1,-1
      ECAL(I)=ECAL(I+1)-DELTAE(ECAL(I+1),ALPHA_SLOPE,ARHO,K_LSS)
C      WRITE(99,*) I,ECAL(I),SLOPE*(I-.5)+PHD
      IF (ECAL(I) .LT.0) THEN
         ECAL(I) = 0.0d0
         GOTO 150
      ENDIF
140      CONTINUE
C
150      CONTINUE
C
C --- Calculate E for 20 channels above surface
      I1 = ICHAN+1
      DO 160 I=I1,I1+19
      ECAL(I)=ECAL(I-1)+DELTAE(ECAL(I-1),ALPHA_SLOPE,ARHO,K_LSS)
160      CONTINUE
C
      RETURN
      END

C
C----------------------------------------------------------------
      FUNCTION DELTAE(E,ALPHA_SLOPE,A_RHO,K_LSS)
C----------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION  K_LSS
C
C --- Define constants for Kaufman PHD calculation
C
      A = 6.0d0
      B = 8.0d0
      C = 525.0d0
      D = 1.407d0
C
      DPHD_DCHN = A/(K_LSS*E+B) - A*K_LSS*E/(K_LSS*E+B)**2 +
     1   A_RHO*C*D*((K_LSS*E)**(-D-1.d0))/(1.d0+C*((K_LSS*E)**(-D)))**2
C
      DELTAE = ALPHA_SLOPE/(1.d0 - DPHD_DCHN)

C      write(98,*) E, ALPHA_SLOPE, A_RHO, DPHD_DCHN, DELTAE
C
C      WRITEC
C      PHD = A*E/(K_LSS*E+B) +A_RHO/(K_LSS*(1.0+C*((K_LSS*E)**(-D))))
C
      RETURN
      END

C
C----------------------------------------------------------------
      FUNCTION TH_FERMI(IZ1,IZ2)
C----------------------------------------------------------------
C
C --- This function calculates the Thomas-Fermi screening radius
C --- for atomic chargres IZ1, IZ2
C --- The radius is in cm.
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      Z1 = DBLE(IZ1)
      Z2 = DBLE(IZ2)
      TH_FERMI = 0.8853d0 * .529d-8/(SQRT(Z1)+SQRT(Z2))**(2.d0/3.d0)
      RETURN
      END
C
C
C--------------------------------------------------------------------------
        SUBROUTINE EODERD(FKD,PASN,P1,P2D,DXABS,
     1                                  EIN,ERD,EOD,EFIN,IELE)
C--------------------------------------------------------------------------
C
C  PURPOSE: to create a table of energy to depth conversion  for
C           first iteration (ie. target = monitor element)
C
C  FKD     -> kinematic factor
C  PASN    -> number of ug/cm2 per channel (step size)
C  P1      -> step size PASN corrected for theta-in
C  P2D     -> step size PASN corrected for scattered particle
C  DXABS   -> absorber thickness of scattered particle IELE
C  EIN     -> energy in of the projectile
C  ERD     -> table of projectile energies as a function of depth
C  EOD     -> table of scatttered energies as a function of depth
C  EFIN    -> lower energy limit of calculations
C  IELE    -> element counter
C -------------------------------------------------------------------------
C              *COMMON BLOCKS*
C  /TADEDX/
C      ITABLE(5,30). table containing names of DEDX tables [*4]
C                     (1,I) : projectile entry thru element(I)
C                     (2,I) : detected monitor exit thru element(I)
C                     (3,I) : detected element exit thru element(I)
C                     (4,I) : detected element thru absorber
C                     (5,I) : detected monitor thru absorber
C      ZTABLE(2,5,30). table containing Z1, Z2 correspoinding to ITABLE elements
C      MTABLE(5,30).   table containing M1 correspoinding to Z1 in ITABLE elements
C
C -------------------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ERD(500),EOD(500)
C
      DOUBLE PRECISION  DXABS
C
CC      ... IL FAUT FAIRE LE CALCUL PAS A PAS ...
C
      DO 31 K = 1,500 ! Re-zero arrays to remove data
      ERD(K) = 0.d0     ! from last call of this routine
31      EOD(K) = 0.d0
C
      ELIM=EIN*FKD/500.0d0
      IF(EFIN.GT.ELIM) ELIM=EFIN
C
C      ... CALCUL A LA SURFACE ...
C
      ERD(1)=EIN
      EO=EIN*FKD
C
C      ... PASSAGE A TRAVERS L'ABSORBANT ...
C
      IF(DXABS .NE. 0) THEN
CC allways the case        IF(ITAB(1:1) .EQ. 'Z') THEN  ! Zieg uses DEDX
          NABS = DXABS/PASN  ! Number of DEDX steps.
          IF(NABS .GT. 1000) THEN  ! If thick, use 1000 steps
            NABS = 1000
            PA = DXABS/1000.d0
C            NABS = NABS/10
C            PA = PASN * 10.
          ELSE                     ! If thin, use normal step size
            PA = PASN
          ENDIF
C
C --- Now deduce the energy lost in the absorber with DEDX
          EX = EO
          DO 50 KK = 1,NABS
50          EX = EX - DEDX(EX,4,IELE) * PA
          EOD(1) = EX
CC        ENDIF
      ELSE   ! If no absorber, no energy is lost.
        EOD(1) = EO
      ENDIF
      I=1
C
C      ... CALCUL PAS A PAS ...
C
60      I=I+1
C
C      ... ON S'ENFONCE DANS L'ECHANTILLON ...
C
      EREAC=ERD(I-1)
      ERD(I)=EREAC - DEDX(EREAC,1,1)*P1
C
C      ... ON REMONTE VERS LA SURFACE ...
C
      IVS=I
      EO=FKD*ERD(IVS) - DEDX(FKD*ERD(IVS),3,IELE)*P2D
      IF(IVS.GT.2) THEN
70            IVS=IVS-1
            EO=EO - DEDX(EO,3,IELE)*P2D
            IF(IVS.GT.2) GO TO 70
      ENDIF
C
C      ... PASSAGE A TRAVERS L'ABSORBANT ...
C
      IF(DXABS .NE. 0) THEN
CC now allways the case   IF(ITAB(1:1) .EQ. 'Z') THEN  ! Zieg uses DEDX
          NABS = DXABS/PASN  ! Number of DEDX steps.
          IF(NABS .GT. 1000) THEN  ! If thick, use 1000 steps
            NABS = 1000
            PA = DXABS/1000.d0
C            NABS = NABS/10
C            PA = PASN * 10.
          ELSE                     ! If thin, use normal step size
            PA = PASN
          ENDIF
C
C --- Now deduce the energy lost in the absorber with DEDX
          EX = EO
          DO 75 KK = 1,NABS
          IF(EX .LE. 0) GO TO 80
75          EX = EX - DEDX(EX,4,IELE) * PA
          EOD(I) = EX
CC        ENDIF
      ELSE   ! If no absorber, no energy is lost.
        EOD(I) = EO
      ENDIF
      IF(EOD(I).GT.ELIM.AND.I.LT.499) GO TO 60
C
80      IF(EOD(I).GT.ELIM) THEN
            WRITE(11,120) EOD(I),ELIM
120            FORMAT(/10X,'EODERD TABLE FILLED.',/10X,
     +      'EOD(499)= ',F6.3,' GREATER THAN ELIM= ',F6.3)
      ENDIF
C
C
      DO 121 K=I,500
      ERD(K)=0.0d0
121      EOD(K)=0.0d0
C
      RETURN
      END
C
C
C
C
C------------------------------------------------------------------C
      SUBROUTINE ERDX1(FKM,EOUT,EREAC,DX)
C------------------------------------------------------------------C
C             CETTE SOUS ROUTINE CALCULE LA PROFONDEUR             C
C             ET L'ENERGIE ECHANGEE D'UNE INTERACTION              C
C------------------------------------------------------------------C
C  FKM   -> kinematic factor
C  EOUT  -> energy of scattered particle
C  EREAC -> `corrected' input energy
C  DX    -> depth (channels) corresponding to EIN & EOUT
C------------------------------------------------------------------C
C              *COMMON BLOCKS*
C  /C/
C      RETA1...... angle of incidence (theta-in) [radians]
C      RETA2M..... monitor detector angle [radians]
C      RETA2D(30). exit angle of detected element [radians]
C      P1......... step size (PASN) corrected for RETA1 [ug/cm2]
C      P2M........ step size corrected for RETA2M [ug/cm2]
C      P2D(30).... step size corrected for RETA2D [ug/cm2]
C      PASN....... step size [ug/cm2]
C      EIN........ energy of projectile [MeV]
C      EFIN....... lower energy limit [MeV]
C------------------------------------------------------------------
C    
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION RETA2D(30),P2D(30)
      COMMON/C/RETA1,RETA2M,RETA2D,P1,P2M,P2D,PASN,EIN,EFIN
C
      ERI=EIN
C
      NI=1
      ERM=EOUT
C
C            ... ENTREE DANS LA CIBLE ...                                
C
10      ERMM=ERM
      ERIO=ERI
      ERM=ERM+DEDX(ERM,2,1)*P2M
      ERI=ERI-DEDX(ERI,1,1)*P1
C                        if(MOD(NI,50).eq.0)
      IF(ERM/FKM.LT.ERI)THEN
            NI=NI+1
            IF(NI.LT.500)THEN
                  GOTO 10                   !PAS ASSEZ PROFOND
            ELSE
                  EREAC=-1.0d0      !FLAG D'ERREUR
                  RETURN
            ENDIF
      ELSE
            IF(ERM/FKM.GT.ERI)THEN
                  NI=NI-1
                  IF(NI.GT.0)THEN
                  DEDX1=DEDX(ERIO,1,1)
                  DEDX2=DEDX(ERMM,2,1)
                  DEDX1=DEDX1/SIN(RETA1)
                  DEDX2=DEDX2/SIN(RETA2M)
                     FRAC=(ERIO-ERMM/FKM)/
     +                  (DEDX1+DEDX2/FKM)
                  ELSE
                        EREAC=-2.0d0      !FLAG D'EREUR
                        RETURN
                  ENDIF
            ELSE                                    !ERM/FKM=ERI
                  FRAC=0.0d0
                  ERIO=ERI
            ENDIF
      ENDIF
C
C            ... PROFONDEUR DE COLLISION ...
C
      DX=NI+FRAC/PASN
      EREAC=ERIO-FRAC*DEDX1
      RETURN
      END
C
C
C
C
C------------------------------------------------------------------C
      SUBROUTINE ERDX2(FKM,EOUT,EREAC,DX)
C------------------------------------------------------------------C
C             CETTE SOUS ROUTINE CALCULE LA PROFONDEUR             C
C             ET L'ENERGIE ECHANGEE D'UNE INTERACTION              C
C            DANS LE CAS D'UNE ITERATION                          C
C------------------------------------------------------------------C
C  FKM   -> kinematic factor
C  EOUT  -> energy of scattered particle
C  EREAC -> `corrected' input energy
C  DX    -> depth (ug/cm2) corresponding to EIN & EOUT
C -----------------------------------------------------------------
C              *COMMON BLOCKS*
C  /C/
C      RETA1...... angle of incidence (theta-in) [radians]
C      RETA2M..... monitor detector angle [radians]
C      RETA2D(30). exit angle of detected element [radians]
C      P1......... step size (PASN) corrected for RETA1 [ug/cm2]
C      P2M........ step size corrected for RETA2M [ug/cm2]
C      P2D(30).... step size corrected for RETA2D [ug/cm2]
C      PASN....... step size [ug/cm2]
C      EIN........ energy of projectile [MeV]
C      EFIN....... lower energy limit [MeV]
C-------------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION RETA2D(30),P2D(30)
      COMMON/C/RETA1,RETA2M,RETA2D,P1,P2M,P2D,PASN,EIN,EFIN
C
      ERI=EIN
C
      NI=1
      ERM=EOUT
C
C            ... ENTREE DANS LA CIBLE ...                                
C
10      ERMM=ERM
      ERIO=ERI
      ERM=ERM+DEDXIT(ERM,2,NI+5)*P2M
      ERI=ERI-DEDXIT(ERI,1,NI+5)*P1
      IF(ERM/FKM.LT.ERI)THEN
            NI=NI+1
            IF(NI.LT.500)THEN
                  GOTO 10                   !PAS ASSEZ PROFOND
            ELSE
                  EREAC=-1.0d0      !FLAG D'ERREUR
                  RETURN
            ENDIF
      ELSE
            IF(ERM/FKM.GT.ERI)THEN
                  NI=NI-1
                  IF(NI.GT.0)THEN
                  DEDX1=DEDXIT(ERIO,1,NI+5)
                  DEDX2=DEDXIT(ERMM,2,NI+5)
                  DEDX1=DEDX1/SIN(RETA1)
                  DEDX2=DEDX2/SIN(RETA2M)
                     FRAC=(ERIO-ERMM/FKM)/
     +                  (DEDX1+DEDX2/FKM)
                  ELSE
                        EREAC=-2.0d0      !FLAG D'EREUR
                        RETURN
                  ENDIF
            ELSE                                    !ERM/FKM=ERI
                  FRAC=0.0d0
                  ERIO=ERI
            ENDIF
      ENDIF
C
C            ... PROFONDEUR DE COLLISION ...
C
      DX=NI+FRAC/PASN
      EREAC=ERIO-FRAC*DEDX1
      RETURN
      END
C
C
C
C
C------------------------------------------------------------
      SUBROUTINE ITERAD(ISUC,IELE,FKD,ITT,NIT,SIGMIT,RATIO)
C------------------------------------------------------------
C
C  PURPOSE: to create a table of energy to depth conversion  for
C           multiple element target. Then calls routine ABONDE to
C           convert energy spectra to profile spectra
C
C  ISUC  -> success flag
C  IELE  -> element identifier
C  FKD   -> kinematic factor
C  ITT   -> iteration number
C  NIT   -> number of iterations requested
C  RATIO -> ratio of solid angle between secondary/principle detectors
C -----------------------------------------------------------
C              *COMMON BLOCKS*
C  /C/
C      RETA1...... angle of incidence (theta-in) [radians]
C      RETA2M..... monitor detector angle [radians]
C      RETA2D(30). exit angle of detected element [radians]
C      P1......... step size (PASN) corrected for RETA1 [ug/cm2]
C      P2M........ step size corrected for RETA2M [ug/cm2]
C      P2D(30).... step size corrected for RETA2D [ug/cm2]
C      PASN....... step size [ug/cm2]
C      EIN........ energy of projectile [MeV]
C      EFIN....... lower energy limit [MeV]
C
C  /TADEDX/
C      ITABLE(5,30). table containing names of DEDX tables [*4]
C                     (1,I) : projectile entry thru element(I)
C                     (2,I) : detected monitor exit thru element(I)
C                     (3,I) : detected element exit thru element(I)
C                     (4,I) : detected element thru absorber
C                     (5,I) : detected monitor thru absorber
C      ZTABLE(2,5,30). table containing Z1, Z2 correspoinding to ITABLE elements
C      MTABLE(5,30).   table containing M1 correspoinding to Z1 in ITABLE elements
C
C  /IDE/
C      IDECON(1024,30).. profile spectra for target elements
C      MCDET(30)........ mass of target elements [amu]
C
C  /EL/
C       NELEM....... number of elements
C      TOFELE...... number of elements in specified target profile
C
C  /Q/
C      ROMEGA(30).. ratio of solid angles between monitor & element
C                    detectors
C      DXABS(30)... absorber for detected elements
C
C  /M/
C      R(30)....... normalization constant
C      ERMON....... average energy of monitor window (0th iteration)[MeV]
C      DXMON....... thickness of monitor window (0th iteration) [ug/cm2]
C      ED(30)...... energy/channel for element(I)
C      BD(30)...... intercept for element(I)
C      EM.......... energy/channel for monitor
C      BM.......... intercept for monitor
C
C  /IT/
C      ERMONIT..... average energy of monitor (iterated)
C      DXMONIT..... average depth of monitor (iterated)
C
C  /SPEC/
C      IMON....... name of monitor energy spectrum [*22]
C      IDET(30)... name of element energy spectrum [*22]
C      IPRO(30)... name of profile spectra for element (I) [*22]
C
C  /PROM/
C      SIGM........ cross section for monitor 0th iteration
C      INTEG....... number of counts in monitor window
C
C  /A/
C      TABLE(60,5,30).. DEDX tables corresponding to ITABLE(5,30)
C-----------------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER*4 IDECON(1024,30),DET(1024),TOFELE
      DOUBLE PRECISION  MCDET(30)
      CHARACTER*1022 IMON,IDET(30)
      CHARACTER*1022 IPRO(30)
      CHARACTER*30 TEMPS !!,ITAB*16
      CHARACTER ITT_OLD
C
      DIMENSION ED(30),BD(30),ERD(500),EOD(500),R(30),DXABS(30),
     1 ROMEGA(30),RETA2D(30),P2D(30)
CC      COMMON/A/TABLE,NBRKS(5,30)
C
      COMMON/IDE/IDECON,MCDET
      COMMON/M/R,ERMON,DXMON,ED,BD,EM,BM
      COMMON/C/RETA1,RETA2M,RETA2D,P1,P2M,P2D,PASN,EIN,EFIN
      COMMON/SPEC/IMON,IDET,IPRO
      COMMON/Q/ROMEGA,DXABS
      COMMON/IT/ERMONIT,DXMONIT
      COMMON/EL/NELEM,TOFELE
      COMMON/PROM/SIGM,INTEG
C
      CALL DISQUE(IDET(IELE),DET,1,ISUC,0)
      IF(ISUC.NE.1) THEN
        WRITE(11,201)
        WRITE(11,300)
201        FORMAT(/10X,'DETECTOR SPECTRUM MISSING; ELEMENT SHOULD',
     1        /10X,'HAVE BEEN DROPPED FROM LIST IN PROFILD;')
300        FORMAT(/10X,'GOING TO NEXT ELEMENT')
      ELSE
          CALL CTIME(TIME8(), TEMPS)
          WRITE(11,205)TEMPS,IDET(IELE)
      ENDIF

205   FORMAT(/10X,A30,/10X,' DETECTOR SPECTRUM ',A22,' HAS BEEN READ')
C
C --- If element is TOF hydrogen, correct energy spectrum
C     as a function of DEDX at the energy entering absorber.
C
C      IF (IDET(IELE)(3:4) .EQ. 'HX' .AND. DETD .EQ. 'P') THEN
C        WRITE(6,'('' AAARRRGGG!!!!'')')
C        DO 30 I=1,1024
C        E=ED(IELE)*I+BD(IELE)
C        IF(DET(I) .EQ. 0 .OR. E .LE. EFIN) GOTO 30
C
C --- Correct for absorber and calculate DEDX
C        IF(DXABS(IELE).EQ.0.0) THEN
C          D = 1.    !DEDX
C        ELSE
CC now allways the case    IF(ITAB(1:1) .EQ. 'Z') THEN  ! Zieg uses DEDX
C            NABS = DXABS(IELE)/PASN
C            IF(NABS .GT. 1000) THEN
C              NABS = NABS/10
C              PA = PASN * 10.
C            ELSE
C              PA = PASN
C            ENDIF
C            EX = E
C            DO 51 KK = 1,NABS
C51            EX = EX + DEDX(EX,4,IELE) * PA
C            E2 = EX
Cc            D = DEDX(E2,4,1)
CC          ENDIF
C        ENDIF
C
C***        EFF = (2.2222*D+.15)/RATIO
C        EFF = (2.4290*D+.103)/RATIO   !MJ02 -Changed Aug 28/1992
C        DET(I) = DET(I)/EFF
C30        CONTINUE
C      ENDIF
C
C --- Re-zero arrays to remove data from last call of this routine
C
      DO 31 K = 1,500
      ERD(K) = 0.d0
31      EOD(K) = 0.d0
C
      ELIM=EIN*FKD/500.d0
      IF(EFIN.GT.ELIM) ELIM=EFIN
C
C             ...CALCUL A LA SURFACE...
C
      ERD(1)=EIN
      EO=EIN*FKD
C
C             ...PASSAGE A TRAVERS L'ABSORBANT...
C
      IF(DXABS(IELE) .NE. 0) THEN
CC        IF(ITAB(1:1) .EQ. 'Z') THEN  ! Zieg uses DEDX
          NABS = DXABS(IELE)/PASN  ! Number of DEDX steps.
          IF(NABS .GT. 1000) THEN  ! If thick, use 1000 steps
            NABS = 1000
            PA = DXABS(IELE)/1000.d0
C            NABS = NABS/10
C            PA = PASN * 10.
          ELSE                     ! If thin, use normal step size
            PA = PASN
          ENDIF
C
C --- Now deduce the energy lost in the absorber with DEDX
          EX = EO
          DO 50 KK = 1,NABS
C              write(11,*) 'b) kk, ex:', kk, ex
50          EX = EX - DEDX(EX,4,IELE) * PA
          EOD(1) = EX
CC        ENDIF
      ELSE   ! If no absorber, no energy is lost.
        EOD(1) = EO
      ENDIF
      I=1
C
C            ...CALCUL PAS A PAS...
C
60      I=I+1
      EREAC=ERD(I-1)
      ERD(I)=EREAC - DEDXIT(EREAC,1,I+5)*P1
      IVS=I
      EO=ERD(IVS)*FKD
      EO=EO - DEDXIT(EO,3,IVS+5)*P2D(IELE)
      IF(IVS.GT.2) THEN
70            IVS=IVS-1
            EO=EO - DEDXIT(EO,3,IVS+5)*P2D(IELE)
            IF(IVS.GT.2) GO TO 70
      ENDIF


      IF(DXABS(IELE) .NE. 0) THEN
CC        IF(ITAB(1:1) .EQ. 'Z') THEN  ! Zieg uses DEDX
          NABS = DXABS(IELE)/PASN  ! Number of DEDX steps.
          IF(NABS .GT. 1000) THEN  ! If thick, use 1000 steps
            NABS = 1000
            PA = DXABS(IELE)/1000.d0
C            NABS = NABS/10
C            PA = PASN * 10.
          ELSE                     ! If thin, use normal step size
            PA = PASN
          ENDIF
C
C --- Now deduce the energy lost in the absorber with DEDX
          EX = EO
          DO 75 KK = 1,NABS
          IF(EX .LE. 0) GO TO 80
75          EX = EX - DEDX(EX,4,IELE) * PA
          EOD(I) = EX
CC        ENDIF
      ELSE   ! If no absorber, no energy is lost.
        EOD(I) = EO
      ENDIF
      IF(EOD(I).GT.ELIM.AND.I.LT.499) GO TO 60
C
80      IF(EOD(I).GT.ELIM) THEN
            WRITE(11,500) EOD(I),ELIM
500            FORMAT(/10X,'EODERD TABLE FILLED.',
     +      /10X,'EOD(499)= ',F6.3,' GREATER THAN ELIM= ',F6.3)
      ENDIF
C
      DO 81 J=I,500
      EOD(J)=0.d0
81      ERD(J)=0.d0
C
C
      RR=R(IELE)*(SIGMIT/SIGM)*(DXMONIT/DXMON) ! adjust normalization
      CALL ABONDE(RR,ERD,EOD,IDECON(1,IELE),DET,ED(IELE),BD(IELE),
     1     RETA1,RETA2D(IELE),IELE)
C      WRITE(11,5000) (IDECON(MM,IELE), MM=1,500)
C5000      FORMAT(10(I7,1X))
C
C --- Deduce last character in name of profile spectrum. For regular
C     iteration procedure, last character is iteration number (0 for
C     original profile). For NIT = -1, last character is "P" for old
C     profile, for NIT = -2, last character is "C" for composition, and
C     for NIT = -3, last character is "S" for strata.
C
      IF (NIT .GT. 0) THEN
        ITT_OLD = IPRO(IELE)(1:1)
        WRITE(IPRO(IELE)(1:1),6000) ITT
6000        FORMAT(I1)
      ELSEIF(NIT .EQ. -1) THEN
        IPRO(IELE)(1:1) = 'P'
      ELSEIF(NIT .EQ. -2) THEN
        IPRO(IELE)(1:1) = 'C'
      ELSEIF(NIT .EQ. -3) THEN
        IPRO(IELE)(1:1) = 'S'
      ENDIF
C
C --- Write new profile to disk
      CALL DISQUE(IPRO(IELE),IDECON(1,IELE),2,ISUC,1)
        CALL CTIME(TIME8(), TEMPS)
      WRITE(11,210) TEMPS,IPRO(IELE)
210      FORMAT(/10X,A30,/5X,'ITERATED PROFILE SPECTRUM ',
     1      A22,' HAS BEEN WRITTEN')
C
C --- For regular iteration procedure, read profile from last
C     iteration back into memory. Thus, for all elements, the
C     substrate composition is deduced from the same iteration.
      IF(TOFELE .EQ. 0) THEN
        IPRO(IELE)(1:1) = ITT_OLD
        CALL DISQUE(IPRO(IELE),IDECON(1,IELE),1,ISUC,0)
      ENDIF
C
      RETURN
      END
C
C
C
C
C--------------------------------------------------------------
      SUBROUTINE ITERAM(ISUC,P1)
C--------------------------------------------------------------
C  PURPOSE: to calculate the average energy and depth of monitor 
C           window with updated profile spectra
C
C  ISUC  -> success flag
C  P1    -> step size (PASN) corrected for theta-in
C -------------------------------------------------------------
C              *COMMON BLOCKS*
C  /N/
C      FKM......... monitor kinematic factor
C      ESUPM1...... upper energy for monitor [MeV] (corrected for
C      EINFM1...... lower energy for monitor [MeV]  .. absorber)
C
C  /IT/
C      ERMONIT..... average energy of monitor (iterated)
C      DXMONIT..... average depth of monitor (iterated)
C
C  /MONITOR/
C      MINF........ lower depth channel for monitor depth
C      MSUP........ upper depth channel for monitor depth
C-----------------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/N/FKM,ESUPM1,EINFM1
      COMMON/IT/ERMONIT,DXMONIT
      COMMON/MONITOR/MINF,MSUP
C
C
        ISUC = 1
      DINF=0.d0
      DSUP=0.d0
      CALL ERDX2 (FKM,ESUPM1,ERSUP,DSUP)
      IF (ERSUP.EQ.-1.0) THEN
            WRITE(11,201)
            WRITE(11,250)
201            FORMAT(/10X,'UPPER ENERGY TOO LOW IN ERDX2;',
     +      ' NORMALIZATION ERROR;')
250      FORMAT(/10X,'ABANDONING ITERATION PROCEDURE')
            ISUC=0
            RETURN
      ELSEIF (ERSUP.EQ.-2.0) THEN
            WRITE(11,202)
            WRITE(11,250)
202            FORMAT(/10X,'UPPER CHANNEL TOO HIGH IN ERDX2;')
            ISUC=0
            RETURN
      ENDIF
C
C
      CALL ERDX2(FKM,EINFM1,ERINF,DINF)
      IF (ERINF.EQ.-1.0) THEN
            WRITE(11,203)
            WRITE(11,250)
203            FORMAT(/10X,'LOWER ENERGY TOO LOW IN ERDX2;',
     +      ' NORMALIZATION ERROR;')
            ISUC=0
            RETURN
      ENDIF
C
C
      ERMONIT=(ERSUP+ERINF)/2.0d0
      DXMONIT=(DINF-DSUP)*P1
C
C --- Deduce depth channel for upper and lower monitor windows.
      MINF = NINT(DINF + 6.d0)  ! 5 channels above surface plus
      MSUP = NINT(DSUP + 6.d0)  ! 1 channel to put intercept at
      RETURN                  ! channel 1 instead of 0. See SURF.
      END
C
C
C---------------------------------------------------------------------
      SUBROUTINE PROFILD (ISUC,IELE,FKD, SURFE)
C---------------------------------------------------------------------
C
C  PURPOSE:  profile the elements for iteration 0
C
C  ISUC   -> success flag (=1 success)
C  IELE   -> element array number
C  FKD    -> kinematic factor
C --------------------------------------------------------------------
C              *COMMON BLOCKS*
C  /TADEDX/
C      ITABLE(5,30). table containing names of DEDX tables [*4]
C                     (1,I) : projectile entry thru element(I)
C                     (2,I) : detected monitor exit thru element(I)
C                     (3,I) : detected element exit thru element(I)
C                     (4,I) : detected element thru absorber
C                     (5,I) : detected monitor thru absorber
C      ZTABLE(2,5,30). table containing Z1, Z2 correspoinding to ITABLE elements
C      MTABLE(5,30).   table containing M1 correspoinding to Z1 in ITABLE elements
C
C  /C/
C      RETA1...... angle of incidence (theta-in) [radians]
C      RETA2M..... monitor detector angle [radians]
C      RETA2D(30). exit angle of detected element [radians]
C      P1......... step size (PASN) corrected for RETA1 [ug/cm2]
C      P2M........ step size corrected for RETA2M [ug/cm2]
C      P2D(30).... step size corrected for RETA2D [ug/cm2]
C      PASN....... step size [ug/cm2]
C      EIN........ energy of projectile [MeV]
C      EFIN....... lower energy limit [MeV]
C
C  /IDE/
C      IDECON(1024,30).. profile spectra for target elements
C      MCDET(30)........ mass of target elements [amu]
C
C  /M/
C      R(30)....... normalization constant
C      ERMON....... average energy of monitor window (0th iteration)[MeV]
C      DXMON....... thickness of monitor window (0th iteration) [ug/cm2]
C      ED(30)...... energy/channel for element(I)
C      BD(30)...... intercept for element(I)
C      EM.......... energy/channel for monitor
C      BM.......... intercept for monitor
C
C  /D/
C      MP......... mass of projectile [amu]
C      MCM........ mass of monitor [amu]
C      MDM........ mass of detected monitor (MCM if ERD, MP if RBS)
C      MCD........ mass of target element [amu]
C      MDD........ mass of detected element (MP if RBS, MCD if ERD)
C      IZP........ Z of projectile
C      IZM........ Z of monitor
C       IZDM....... Z of detected monitor (IZP if RBS, IZM if ERD)
C      IZD........ Z of target element
C      IZDD....... Z of detected element (IZP if RBS, IZD if ERD)
C
C  /PROM/
C      SIGM........ cross section for monitor 0th iteration
C      INTEG....... number of counts in monitor window
C
C  /SPEC/
C      IMON....... name of monitor energy spectrum [*22]
C      IDET(30)... name of element energy spectrum [*22]
C      IPRO(30)... name of profile spectra for element (I) [*22]
C
C  /Q/
C      ROMEGA(30).. ratio of solid angles between monitor & element
C                    detectors
C      DXABS(30)... absorber for detected elements
C
C  /CAL/
C       ECAL(1024,30).... Energy calibration for element(I)
C                         The value of ECAL is the UPPER edge of the
C                         channel
C       ECALM(1024)...... Energy calibration of monitor
C       ICHN_SURF(30)..... Surface channel for element (I)
C      ARHOD(30)........ Constant for PHD calculation for element(I)
C      ARHOM............ Constant for PHD calculation for monitor
C       ALPHA_SLOPE...... Slope of alpha particle energy calibration
C--------------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER*4 IDECON(1024,30),DET(1024)
      DOUBLE PRECISION  MP,MCM,MDM,MCD,MDD,MCDET(30),SURFE
      CHARACTER*30 TEMPS
      CHARACTER*1022 IDET(30),IMON
      CHARACTER*1022 IPRO(30)
C
      DIMENSION R(30),ED(30),BD(30),RETA2D(30),P2D(30),
     1         ERD(500),EOD(500),DXABS(30),ROMEGA(30),ECAL(1024,30),
     1         ECALM(1024),ARHOD(30),ICHN_SURF(30)
C
      COMMON/IDE/IDECON,MCDET
      COMMON/M/R,ERMON,DXMON,ED,BD,EM,BM
      COMMON/C/RETA1,RETA2M,RETA2D,P1,P2M,P2D,PASN,EIN,EFIN
      COMMON/SPEC/IMON,IDET,IPRO
      COMMON/D/MP,MCM,MDM,MCD,MDD,IZP,IZM,IZDM,IZD,IZDD
      COMMON/PROM/SIGM,INTEG
      COMMON/Q/ROMEGA,DXABS
      COMMON/CAL/ECAL,ECALM,ICHN_SURF,ARHOD,ARHOM,ALPHA_SLOPE
C
C
      ED(IELE)=ED(IELE)/1000.d0  ! Convert to MeV
      BD(IELE)=BD(IELE)/1000.d0
C
      CALL DISQUE (IDET(IELE),DET,1,ISUC,0) ! Read in energy spectrum.

      IF(ISUC.NE.1) THEN
        WRITE(11,204)
        WRITE(11,300)
204        FORMAT(/10X,'DETECTOR SPECTRUM UNAVAILABLE;')
300        FORMAT(/10X,'EXITING')
        RETURN
      ELSE
          CALL CTIME(TIME8(), TEMPS)
        WRITE(11,205) TEMPS,IDET(IELE)
205     FORMAT(/10X,A30,/10X,' DETECTOR SPECTRUM ',A22,' HAS BEEN READ')
      ENDIF
C
      FKD=CINEM(MP,MCD,MDD,RETA1,RETA2D(IELE)) ! Deduce kinematic factor
C
C --- Deduce table of EOD and ERD values
      CALL EODERD(FKD,PASN,P1,P2D(IELE),DXABS(IELE),
     1           EIN,ERD,EOD,EFIN,IELE)
        SURFE = EOD(1)
C
C --- Calculate normalization factor (sigma/intensity)
      R(IELE)=SIGM*(DXMON/P1)/INTEG * ROMEGA(IELE)
C
C --- Calculate energy calibration
C     NOTE: IZDD = IZD (ERD), IZP (RBS) , Used in ENERGY_CAL
C
      CALL ENERGY_CAL(ALPHA_SLOPE,IELE,IZDD,MDD,ED(IELE),BD(IELE),
     1  FKD,DXABS(IELE),4,ARHOD(IELE),ECAL(1,IELE),ICHN_SURF(IELE))
C
C --- Create profile spectrum
      IPRO(IELE)(1:1) = '0' ! Iteration zero
      CALL ABONDE(R(IELE),ERD,EOD,IDECON(1,IELE),
     1           DET,ED(IELE),BD(IELE),RETA1,RETA2D(IELE),IELE)
      CALL DISQUE(IPRO(IELE),IDECON(1,IELE),2,I,1)
C
        CALL CTIME(TIME8(), TEMPS)
      WRITE(11,208) TEMPS,IPRO(IELE)
208   FORMAT(/10X,A30,/10X,' PROFILE SPECTRUM ',A22,' HAS BEEN WRITTEN')
      RETURN
      END
C
C
C
C
C----------------------------------------------------------------
      SUBROUTINE PROFILM (ISUC,BSUP,BINF,DXABSM)
C----------------------------------------------------------------
C
C  PURPOSE: determine average energy & depth of monitor region
C           for monitor element -> monitor element.
C
C  ISUC   -> success flag
C  BSUP   -> upper energy channel of monitor
C  BINF   -> lower energy channel of monitor
C  DXABSM -> width of absorber of monitor
C ---------------------------------------------------------------
C              *COMMON BLOCKS*
C  /TADEDX/
C      ITABLE(5,30). table containing names of DEDX tables [*4]
C                     (1,I) : projectile entry thru element(I)
C                     (2,I) : detected monitor exit thru element(I)
C                     (3,I) : detected element exit thru element(I)
C                     (4,I) : detected element thru absorber
C                     (5,I) : detected monitor thru absorber
C      ZTABLE(2,5,30). table containing Z1, Z2 correspoinding to ITABLE elements
C      MTABLE(5,30).   table containing M1 correspoinding to Z1 in ITABLE elements
C
C  /C/
C      RETA1...... angle of incidence (theta-in) [radians]
C      RETA2M..... monitor detector angle [radians]
C      RETA2D(30). exit angle of detected element [radians]
C      P1......... step size (PASN) corrected for RETA1 [ug/cm2]
C      P2M........ step size corrected for RETA2M [ug/cm2]
C      P2D(30).... step size corrected for RETA2D [ug/cm2]
C      PASN....... step size [ug/cm2]
C      EIN........ energy of projectile [MeV]
C      EFIN....... lower energy limit [MeV]
C
C  /M/
C      R(30)....... normalization constant
C      ERMON....... average energy of monitor window (0th iteration)[MeV]
C      DXMON....... thickness of monitor window (0th iteration) [ug/cm2]
C      ED(30)...... energy/channel for element(I)
C      BD(30)...... intercept for element(I)
C      EM.......... energy/channel for monitor
C      BM.......... intercept for monitor
C
C  /D/
C      MP......... mass of projectile [amu]
C      MCM........ mass of monitor [amu]
C      MDM........ mass of detected monitor (MCM if ERD, MP if RBS)
C      MCD........ mass of target element [amu]
C      MDD........ mass of detected element (MP if RBS, MCD if ERD)
C      IZP........ Z of projectile
C      IZM........ Z of monitor
C       IZDM....... Z of detected monitor (IZP if RBS, IZM if ERD)
C      IZD........ Z of target element
C      IZDD....... Z of detected element (IZP if RBS, IZD if ERD)
C
C  /PROM/
C      SIGM........ cross section for monitor 0th iteration
C      INTEG....... number of counts in monitor window
C
C  /SPEC/
C      IMON....... name of monitor energy spectrum [*22]
C      IDET(30)... name of element energy spectrum [*22]
C      IPRO(30)... name of profile spectra for element (I) [*22]
C
C  /N/
C      FKM......... monitor kinematic factor
C      ESUPM1...... upper energy for monitor [MeV] (corrected for
C      EINFM1...... lower energy for monitor [MeV]  .. absorber)
C
C  /MONITOR/
C      MINF........ lower depth channel for monitor depth
C      MSUP........ upper depth channel for monitor depth
C
C  /CAL/
C       ECAL(1024,30).... Energy calibration for element(I)
C                         The value of ECAL is the UPPER edge of the 
C                         channel
C       ECALM(1024)...... Energy calibration of monitor
C       ICHN_SURF(30)..... Surface channel for element (I)
C      ARHOD(30)........ Constant for PHD calculation for element(I)
C      ARHOM............ Constant for PHD calculation for monitor
C       ALPHA_SLOPE...... Slope of alpha particle energy calibration
C--------------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*1022 IDET(30),IMON
      CHARACTER*1022 IPRO(30)
      CHARACTER*30 TEMPS !!, ITAB*16
      DOUBLE PRECISION  MP,MCM,MDM,MCD,MDD
      INTEGER*4 BUFF(1024)
      DIMENSION ED(30),BD(30),R(30),
     1         RETA2D(30),P2D(30),ECAL(1024,30),ECALM(1024),
     1         ARHOD(30),ICHN_SURF(30)
      INTEGER BSUP,BINF
C
      COMMON/C/RETA1,RETA2M,RETA2D,P1,P2M,P2D,PASN,EIN,EFIN
      COMMON/N/FKM,ESUPM1,EINFM1
      COMMON/SPEC/IMON,IDET,IPRO
      COMMON/D/MP,MCM,MDM,MCD,MDD,IZP,IZM,IZDM,IZD,IZDD
      COMMON/PROM/SIGM,INTEG
      COMMON/M/R,ERMON,DXMON,ED,BD,EM,BM
      COMMON/CAL/ECAL,ECALM,ICHN_SURF,ARHOD,ARHOM,ALPHA_SLOPE
C
      EM=EM/1000.d0  ! Convert to MeV.
      BM=BM/1000.d0
C
      CALL DISQUE(IMON,BUFF,1,ISUC,0)
C
      IF(ISUC.NE.1) THEN
        WRITE(11,204)
204        FORMAT(/10X,'MONITOR UNAVAILABLE; EXITING')
        RETURN
      ELSE
          CALL CTIME(TIME8(), TEMPS)
        WRITE(11,205) TEMPS,IMON
205     FORMAT(/10X,A30,/10X,' MONITOR SPECTRUM ',A22,' HAS BEEN READ')
      ENDIF
C
C --- Integrate counts in monitor window.
      INTEG=0
      DO 100 J=BINF,BSUP
100      INTEG=INTEG+BUFF(J)
C
C
C --- Calculate the kinematic factor for the monitor element.
      FKM=CINEM(MP,MCM,MDM,RETA1,RETA2M) 
C
C
C --- Calculate energy calibration of monitor
C     NOTE: IZDM = IZM (ERD), IZP (RBS) , Used in ENERGY_CAL
C
      CALL ENERGY_CAL(ALPHA_SLOPE,1,IZDM,MDM,EM,BM,FKM,DXABSM,
     1               5,ARHOM,ECALM,ICHN_SURFM)
C
C
C ----------------- UPPER CHANNEL ------------------------------
C
      IF(ARHOM.GT.0) THEN
         ESUPM=ECALM(BSUP)        ! Upper channel energy (MeV).
      ELSE
         ESUPM=(BSUP+0.5d0)*EM + BM ! Upper channel energy (MeV).
      ENDIF
C
      IF(DXABSM.EQ.0.0) THEN  ! Correct for monitor absorber.
            ESUPM1=ESUPM
      ELSE
CC now allways the case  IF(ITAB(1:1) .EQ. 'Z') THEN  ! Zieg uses DEDX
            NABS = DXABSM/PASN
              IF(NABS .GT. 1000) THEN  ! If thick, use 1000 steps
                NABS = 1000
                PA = DXABSM/1000.d0
C                NABS = NABS/10
C                PA = PASN * 10.
            ELSE
              PA = PASN
            ENDIF
            EX = ESUPM
            DO 50 KK = 1,NABS
50            EX = EX + DEDX(EX,5,1) * PA
            ESUPM1 = EX
CC        ENDIF
      ENDIF
C
C --- Find depth of upper channel
      DSUP=0.d0
      CALL ERDX1 (FKM,ESUPM1,ERSUP,DSUP) ! Depth of upper channel
      IF (ERSUP.EQ.-1.0) THEN
        WRITE (11,208)
208        FORMAT(/10X,'UPPER ENERGY TOO LOW; NORMALIZATION ERROR;',
     1       ' EXITING PROGRAM')
        ISUC=0
        RETURN
      ENDIF
      IF(ERSUP.EQ.-2.0) THEN
        WRITE (11,209) BSUP
209        FORMAT(/10X,'UPPER CHANNEL BSUP= ',I3,' TOO HIGH;',
     1       ' EXITING PROGRAM')
        ISUC=0
        RETURN
      ENDIF
C
C ----------------- LOWER CHANNEL ------------------------------
C
      IF (ARHOM.GT.0) THEN
         EINFM=ECALM(BINF-1)      ! Lower channel energy
      ELSE
         EINFM=(BINF-0.5d0)*EM + BM ! Lower channel energy
      ENDIF
      IF(DXABSM.EQ.0.0) THEN  ! Correct for monitor absorber.
            EINFM1=EINFM
      ELSE
CC now allways the case  IF(ITAB(1:1) .EQ. 'Z') THEN  ! Zieg uses DEDX
            NABS = DXABSM/PASN
                 IF(NABS .GT. 1000) THEN  ! If thick, use 1000 steps
                NABS = 1000
                PA = DXABSM/1000.d0
C                NABS = NABS/10
C                PA = PASN * 10.
            ELSE
              PA = PASN
            ENDIF
            EX = EINFM
            DO 75 KK = 1,NABS
75            EX = EX + DEDX(EX,5,1) * PA
            EINFM1 = EX
CC        ENDIF
      ENDIF
      DINF=0.d0
      CALL ERDX1 (FKM,EINFM1,ERINF,DINF) ! Depth of lower channel
      IF (ERINF.EQ.-1.0) THEN
            WRITE (11,211)
211      FORMAT(/10X,'LOWER ENERGY TOO LOW; NORMALIZATION ERROR;',
     +             ' EXITING PROGRAM')
            ISUC=0
            RETURN
      ENDIF
C
C
      ERMON=(ERSUP+ERINF)/2.d0 ! Average energy of monitor window
      DXMON=(DINF-DSUP)*P1  ! Thickness of monitor window
C
      SIGM=SIG(MP,MCM,MDM,IZP,IZM,RETA1,RETA2M,ERMON)
C
C --- Deduce depth channel for upper and lower monitor windows.
      MINF = NINT(DINF + 6.d0)  ! 5 channels above surface plus
      MSUP = NINT(DSUP + 6.d0)  ! 1 channel to put intercept at
      RETURN                  ! channel 1 instead of 0. See SURF.
      END
C
C
C
C
C------------------------------------------------------------------C
C                           FONCTION RRAN                          C

CC Removed: used only for N & S stopping power

C
C
C------------------------------------------------------------------C
      FUNCTION SIG(MP,MC,MD,IZ1,IZ2,RETA1,RETA2,E)
C------------------------------------------------------------------C
C              CETTE FONTION CALCULE LA SECTION EFFICACE           C
C              DE L'INTERACTION. [mb/sr]                           C
C------------------------------------------------------------------C
C  MP    -> mass of projectile [amu]
C  MC    -> mass of element [amu]
C  MD    -> mass of detected element (MC if ERD, MP if RBS)
C  IZ1   -> Z of projectile
C  IZ2   -> Z of target element
C  RETA1 -> entry angle [rad]
C  RETA2 -> exit angle [rad]
C  E     -> energy of projectile [MeV]
C ------------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        
      DOUBLE PRECISION  MP,MC,MD
      S2=DBLE(IZ1)*DBLE(IZ2)
      IF(ABS(MP-MD).LE.0.1)THEN
C
C            ... CAS RBS ...                                           
C
        S1=SQRT(1-(SIN(RETA1+RETA2)*MP/MC)**2)
        SIG=5.184d0*S2**2*(S1+COS(RETA1+RETA2))
     +          **2/(S1*SIN(RETA1+RETA2)**4)/E**2
      ELSE
C
C            ... CAS ERD ...
C
C --- Special case for He->HX (ie: normal Rutherford cross section doesn't
C     work)  
C
C Use polynomial fit to data from:
C  J. Tirara, F. Bodart
C  LARN, Facultes Universitaires Notre_Dame de la Paix (Belgium)
C  (Submitted for publication in NIM , 1992/93)
C *****
C
C --- Special case for He->HY 
C
C Use polynomial fit to data from:
C  A.J. Kellock, J.E.E. Baglin
C  IBM Almaden Research Center, San Jose, Ca
C *****
C
C --- Valid only for total angle of 30 degrees
C *** Effective: Jan 8/1993 ***
C
        IF (IZ1.EQ.2 .AND. IZ2.EQ.1) THEN
          IF (E .LT. 1.) THEN
            S3=(MP+MC)/MC
            SIG=5.184d0*(S3*S2)**2/(COS(RETA1+RETA2))**3/E**2
          ELSE
            IF (ABS(MC-1.0d0) .LT. 0.1d0) THEN  !-- HX
          SIG = EXP(0.4779d0*E + 2.4758d0 + 5.7117d0/E - 1.8897d0/E**2)
            ELSE  !-- HY
              SIG = EXP(0.58d0*E + 2.99d0 + 4.94d0/E - 2.04d0/E**2)
            ENDIF
          ENDIF
C
C************************************************************************
C Use polynomial fit to data from:
C  Baglin et al., 10th International Conference on Ion Beam Analysis,
C  Eindhoven, The Netherlands, July 1991.
C  (Submitted for publication in NIM B, 1991/92)
C *** Effective: Aug 24/1992 to Jan 8/93 ***
C            SIG = EXP(0.133*E + 4.383 + 2.196/E - 0.042/E**2)
C************************************************************************
C
C************************************************************************
C Use 8-degree polynomial fit to data from:
C  Wang Hong, NIM B34 (1988) 145-151, &
C  Wang Yansen, NIM B17 (1986) 11-14
C  --- valid only for 30 degree total angle
C
C            SIG = 6450.1-0.12776E+5*E+0.11829E+5*E**2-0.62088E+4*E**3
C      1     +0.19876E+4*E**4-0.39428E+3*E**5+0.47219E+2*E**6
C      2     -0.31208E+1*E**7+0.87223E-1*E**8
C************************************************************************
C
C************************************************************************
C Use emperical formula from Szilagyi et al. NIM B43 (1989) 502
C          ANGLE = (RETA1+RETA2)*57.2957795
C          SIG=89.1*E**2-0.119*ANGLE**2-455.7*E+10.2*ANGLE-
C      1        1.38*E*ANGLE+693.
C***********************************************************************

C --- Rutherford cross section
        ELSE
          S3=(MP+MC)/MC
          SIG=5.184d0*(S3*S2)**2/(COS(RETA1+RETA2))**3/E**2
        ENDIF
      ENDIF
      RETURN
      END
C
C------------------------------------------------------------------C
      FUNCTION SURF1(EO1,EO2,DET,E,ICHN1)
C------------------------------------------------------------------C
C
C***** REPLACES SURF FOR Z>11 - INCLUDES ENERGY DEPENDENCE OF PHD *****
C
C             CETTE FONCTION FAIT L'INTEGRALE DU NOMBRE            C
C             DE COUP ENTRE LES ENERGIES EO1 ET EO2 OU             C
C             EO1 EST PLUS GRAND QUE EO2.                          C
C------------------------------------------------------------------C
C  EO1  -> upper energy limit
C  EO2  -> lower energy limit
C  DET  -> energy spectra
C  E -> array of energy calibration (energy of UPPER edge of chn)
C  ICHN1 -> upper limit to search for channel corresponding to E01
C------------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        
      INTEGER*4 DET
      DIMENSION DET(1024),E(1024)
C
C************** CANAUX CORRESPONDANT AUX ENERGIES *****************
C
      DO 10 I=ICHN1,1,-1
10      IF (EO1 .GT. E(I)) GOTO 20
20      ICHN1=I
C
C --- Find energy at chn midpt.
      EMID = (E(I) + E(I+1))/2.d0
C
      IF (EO1 .GE. EMID) THEN
         IF1 = I+1
      ELSE
         IF1 = I
      ENDIF 
      F1 = (EO1-E(I))/(E(I+1)-E(I)) + I + 0.5d0
C
      DO 30 I=ICHN1,1,-1
30      IF (EO2 .GT. E(I)) GOTO 40
40      CONTINUE
C
C --- Find energy at chn midpt.
      EMID = (E(I) + E(I+1))/2.d0
C
      IF (EO2 .GE. EMID) THEN
         IF2 = I+1
      ELSE
         IF2 = I
      ENDIF 
C
      F2 = (EO2-E(I))/(E(I+1)-E(I)) + I + 0.5d0
C
      DF1=IF1+.5d0
      DF2=IF2+.5d0
C
C************** CAS MOINS QUE UN CANAL ****************************
C
      IF(IF1.EQ.IF2)THEN
            IF(F1.LE.DF1.AND.F2.LE.DF1)THEN
                  SURF1=DET(IF1)*(F1-F2)
            ENDIF
            IF(F1.GT.DF1.AND.F2.GT.DF1)THEN
                  SURF1=DET(IF1+1)*(F1-F2)
            ENDIF
            IF(F1.GT.DF1.AND.F2.LE.DF1)THEN
                  SURF1=DET(IF1+1)*(F1-DF1)+DET(IF1)*
     +                  (DF1-F2)
            ENDIF
      ELSE
C
C************* CAS UN CANAL ***************************************
C
            IF(F1.LE.DF1)THEN
                  SOM1=DET(IF1)*(F1-IF1)
            ELSE
                  SOM1=DET(IF1+1)*(F1-DF1)+DET(IF1)/2.d0
            ENDIF
            IF(F2.LT.DF2)THEN
                  SOM2=DET(IF2+1)/2.d0+DET(IF2)*(DF2-F2)
            ELSE
                  SOM2=DET(IF2+1)*(IF2+1-F2)
            ENDIF
            SURF1=SOM1+SOM2
C
C************* CAS DEUX CANAUX ************************************
C
            IF(IF1.NE.IF2+1)THEN
                  SOM1G=DET(IF1)/2.d0
                  SOM2DD=DET(IF2+1)/2.d0
                  SOMP=0.0d0
C
C************* CAS PLUS QUE DEUX CANAUX ***************************
C
                  IF(IF1.NE.IF2+2)THEN
                        INDX=IF1-IF2-2
                        DO 60 J=1,INDX
                              M=IF2+J+1
                              SOMP=DET(M)+SOMP
60                        CONTINUE
                  ENDIF
                  SURF1=SOMP+SOM1G+SOM2DD+SOM1+SOM2
            ENDIF
      ENDIF
      RETURN
      END
C
C
C
C------------------------------------------------------------------C
      FUNCTION SURF(EO1,EO2,DET,ED,BD)
C------------------------------------------------------------------C
C
C***** REPLACED BY SURF1 FOR Z>=11 - INCLUDES ENERGY DEPENDENCE OF PHD *****
C
C             CETTE FONCTION FAIT L'INTEGRALE DU NOMBRE            C
C             DE COUP ENTRE LES ENERGIES EO1 ET EO2 OU             C
C             EO1 EST PLUS GRAND QUE EO2.                          C
C------------------------------------------------------------------C
C  EO1  -> upper energy limit
C  EO2  -> lower energy limit
C  DET  -> energy spectra
C  ED   -> energy/channel
C  BD   -> energy intercept
C------------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      INTEGER*4 DET
      DIMENSION DET(1024)
C
C************** CANAUX CORRESPONDANT AUX ENERGIES *****************
C
      F1 = EO1/ED - BD/ED + 1.0d0
      F2 = EO2/ED - BD/ED + 1.0d0
      IF1=F1
      IF2=F2
      DF1=IF1+.5d0
      DF2=IF2+.5d0
C
C************** CAS MOINS QUE UN CANAL ****************************
C
      IF(IF1.EQ.IF2)THEN
            IF(F1.LE.DF1.AND.F2.LE.DF1)THEN
                  SURF=DET(IF1)*(F1-F2)
            ENDIF
            IF(F1.GT.DF1.AND.F2.GT.DF1)THEN
                  SURF=DET(IF1+1)*(F1-F2)
            ENDIF
            IF(F1.GT.DF1.AND.F2.LE.DF1)THEN
                  SURF=DET(IF1+1)*(F1-DF1)+DET(IF1)*
     +                  (DF1-F2)
            ENDIF
      ELSE
C
C************* CAS UN CANAL ***************************************
C
            IF(F1.LE.DF1)THEN
                  SOM1=DET(IF1)*(F1-IF1)
            ELSE
                  SOM1=DET(IF1+1)*(F1-DF1)+DET(IF1)/2.d0
            ENDIF
            IF(F2.LT.DF2)THEN
                  SOM2=DET(IF2+1)/2.d0+DET(IF2)*(DF2-F2)
            ELSE
                  SOM2=DET(IF2+1)*(IF2+1-F2)
            ENDIF
            SURF=SOM1+SOM2
C
C************* CAS DEUX CANAUX ************************************
C
            IF(IF1.NE.IF2+1)THEN
                  SOM1G=DET(IF1)/2.d0
                  SOM2DD=DET(IF2+1)/2.d0
                  SOMP=0.0d0
C
C************* CAS PLUS QUE DEUX CANAUX ***************************
C
                  IF(IF1.NE.IF2+2)THEN
                        INDX=IF1-IF2-2
                        DO 10 J=1,INDX
                              M=IF2+J+1
                              SOMP=DET(M)+SOMP
10                        CONTINUE
                  ENDIF
                  SURF=SOMP+SOM1G+SOM2DD+SOM1+SOM2
            ENDIF
      ENDIF
      RETURN
      END
C
C
       SUBROUTINE SHOWTABLE
C------------------------------------------------------------------
C  PURPOSE:  Print ITABLE, ZTABLE & MTABLE
C
C -----------------------------------------------------------------
C              *COMMON BLOCKS*
C  /TADEDX/
C      ITABLE(5,30). table containing names of DEDX tables [*4]
C                     (1,I) : projectile entry thru element(I)
C                     (2,I) : detected monitor exit thru element(I)
C                     (3,I) : detected element exit thru element(I)
C                     (4,I) : detected element thru absorber
C                     (5,I) : detected monitor thru absorber
C      ZTABLE(2,5,30). table containing Z1, Z2 correspoinding to ITABLE elements
C      MTABLE(5,30).   table containing M1 correspoinding to Z1 in ITABLE elements
C
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        CHARACTER *4 ITABLE
        INTEGER ZTABLE
        DOUBLE PRECISION  MTABLE
C
        DIMENSION ITABLE(5,30), ZTABLE(2,5,30), MTABLE(5,30)
C
      COMMON/TADEDX/ITABLE, ZTABLE, MTABLE  !!,ITAB
C
        write(11,*) '(I,Z1,Z2,M)TABLE= (1, *) (2,*) (3,*) (4,*) (5,*)'
        do 111 j=1,30
  111   write(11,2111)'>',
     +  (itable(i,j),ZTABLE(1,i,j),ZTABLE(2,i,j),mtable(i,j),i=1,5)
 2111   format(A1,5(4X,A4,2I3,F7.2))
        write(11,*)'--------------'

C --- Flush log file buffered data to disk
        CALL Flush(11)
        RETURN
        END
C
C
C
       FUNCTION IZELEM(ELEM)
C------------------------------------------------------------------
C  PURPOSE:  return Z of element
C
C -----------------------------------------------------------------
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        CHARACTER *2 ELEM
        CHARACTER *2 ATOM

        DIMENSION ATOM(92)

       DATA ATOM/"HX","HE","LI","BE","BX","CX","NX","OX","FX",
     + "NE","NA","MG","AL","SI","PX","SX","CL","AR","KX","CA","SC",
     + "TI","VX","CR","MN","FE","CO","NI","CU","ZN","GA","GE","AS",
     + "SE","BR","KR","RU","SR","YX","ZR","NB","MO","TC","RU","RH",
     + "PD","AG","CD","IN","SN","SB","TE","IX","XE","CS","BA","LA",
     + "CE","PR","ND","PM","SM","EU","GD","TB","DY","HO","ER","TM",
     + "YB","LU","HF","TA","WX","RE","OS","IR","PT","AU","HG","TL",
     + "PB","BI","PO","AT","RN","FR","RA","AC","TH","PA","UX"/
C
       IZELEM = 0

       DO 200 K=1,92
         IF(ELEM.EQ.ATOM(K)) IZELEM = K
 200   CONTINUE

        IF(IZELEM.EQ.0) write(11,*) 'Element unknown: ', ELEM

        RETURN
        END



              FUNCTION STOP96(Z1,MM1,Z2,ENERGY)
Cͻ
C    "STOP96" : Ion Stopping Subroutine used by TRIM                       
C               (C) 1995, J.F.Ziegler, Yorktown,NY,10598                   
C                                                                          
C    Adapted to Fortran77 and simplified                                   
C    by Martin Chicoine, Universit de Montral, 2003                      
C͹
C    return the the stopping power in units of >>> keV/(ug/cm2) <<< !!!!   
C    NUCLEAR STOPPING IS NOT CALCULATED.                                   
C͹
C    ENERGY IS THE ATOM ENERGY                                             
C    EMAX IS THE MAXIMUM ENERGY FOR THE STOPPING LIST                      
C    If M1=0, then M1 is set to the mass of the most abundant isotope      
C͹
C    Call using:                                                           
C                CALL STOPCOEF                                             
C                   ....                                                   
C                Y = STOP96(Z1,M1,Z2,ENERGY)                               
Cͼ
Cͻ
Cͻ array: SCOEF.95 ͹
C> Columns 1-8 : ͼ                    
C    (1=Atomic Number),(2=Mass of MAI),(3=Weight of MAI)  
C    (4=Natural Mass),(5=Target density: g/cm3),          
C    (6=Target Density: atoms/cm3),(7=Fermi Velocity)     
C    (8=Sublimation Energy)                               
C> Columns  9-16 : Proton stopping coefficients.        
C> Columns 17-20 : Proton stopping > 10 MeV.            
C> Columns 21-39 : Ion Lambda values (width of ion)     
C> Columns 40-54 : Tgt Fermi Velocity Correction (<5%)  
C    ** MAI = Most Abundant Isotope                       
C͹
C      Variable definitions for STOP95                    
C>   Z1 = ION ATOMIC NUMBER                             
C>   M1 = ION ATOMIC WEIGHT (AMU)                       
C>   Z2 = TARGET ATOMIC NUMBER                          
C>   M2 = TARGET ATOMIC WEIGHT (AMU)                    
C>   VFERMI = (FERMI VELOCITY OF SOLID) / V0            
C>   SE = CALCULATED ELECTRONIC STOPPING (EV-A2)        
Cͼ

C> The below arrays are defined and filled from the calling program.
C> They are filled using the subroutine STOPCOEF

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/SCOEF96/SCOEF,DPUN

      DOUBLE PRECISION  SCOEF(93,54),ENERGY, M1, MM1, VFERMI, SE

      INTEGER Z1, Z2, DPUN

      M1 = MM1
      IF (M1 .EQ. 0) M1 = SCOEF(Z1,3)     ! Weight (amu) of most abundant isotope.

      VFERMI = SCOEF(Z2,7)             ! VFermi of Target

      IF (Z1 .EQ. 1) CALL PROTON(Z2,ENERGY/M1,SE)            !Energy per mass unit
      IF (Z1 .EQ. 2) CALL HELIUM(Z2,ENERGY/M1,SE)            !Energy per mass unit
      IF (Z1. GT. 2) CALL HEAVY(VFERMI,ENERGY/M1,Z1,Z2,SE)   !Energy per mass unit

      if(DPUN.eq.0) then
          STOP96=SE*SCOEF(Z2,6)/SCOEF(Z2,5)*1.d-24  !This converts Stopping in eV/(1E15-cm2) to keV/(ug/cm2)
      else
         STOP96 = SE*1.0d-2 !This converts Stopping in eV/(1E15-cm2) to keV/(1E16-cm2)   <<<< remark the 16
      endif
C      STOP96 = SE*1.0d-3 !This converts Stopping in eV/(1E15-cm2) to keV/(1E15-cm2)

C      write(11,*)'dedx2:', STOP96, SE*SCOEF(Z2,6)/SCOEF(Z2,5)*1.d-24

      RETURN
      END

C
C Proton Stopping  Powers  
      SUBROUTINE PROTON(Z2,E,SP)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER Z2

      DOUBLE PRECISION  E, SP

      CALL PSTOP (Z2,E,SP)

      RETURN
      END


C
C    HELIUM Stopping Powers    
C---- VELOCITY PROPORTIONAL STOPPING BELOW KEV/AMU  >> HE0 <<.
      SUBROUTINE HELIUM(Z2,E,SH)

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER Z2

      DOUBLE PRECISION  E, SP, HE, HE0, A, HEH, B, SH

      HE0 = 1     !Units = keV/amu

      IF (HE0 .GT. E) THEN
        HE = HE0
        ELSE
        HE = E
      END IF

      B = LOG(HE)
      A = .2865d0+.1266d0*B-.001429d0*B*B+.02402d0*B*B*B-.01135d0*B**4
     $    +.001475d0*B**5
      IF (A .GT. 30) A = 30
      HEH = 1.d0-EXP(-A)
C**** ADD Z1**3 EFFECT TO HE/H STOPPING POWER RATIO** HEH **.
      IF (HE .LT. 1) HE = 1
      A = (1.d0+(.007d0+.00005d0*Z2)*EXP(-(7.6d0-LOG(HE))**2))
      HEH = HEH*A*A

      CALL PSTOP(Z2,HE,SP)

      SH = SP*HEH*4.d0
      IF (E .LE. HE0) SH=SH*SQRT(E/HE0)   !*** CALC. HE VELOCITY PROPORTIONAL STOPPING

      RETURN
      END

C
C HEAVY ION ELECTRONIC STOPPING POWERS 
C**** USE VELOCITY STOPPING FOR (YRMIN=VR/Z1**.67) <= 0.13
C**** OR FOR VR <= 1.0

      SUBROUTINE HEAVY(VFERMI,E,Z1,Z2,SH)

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        COMMON/SCOEF96/SCOEF,DPUN !, SCOEFGAS

      DOUBLE PRECISION  SCOEF(93,54),VFERMI,E,SP,V,YRMIN !,SCOEFGAS(24,11)
     $   ,VRMIN,VR,YR,A,Q,LAMBDA0,LAMBDA1,L,ZETA0,ZETA,SH,EION,
     $   VFCORR0,VFCORR1,VMIN,EEE,HIPOWER

      INTEGER Z1,Z2, DPUN

      YRMIN = 0.13d0
      VRMIN = 1.0d0
      V = SQRT(E/25.d0)/VFERMI    ! Relative Velocity

      IF (V .LT. 1) THEN
         VR = (3.d0 * VFERMI/4.d0)*(1.d0+(2.d0*V*V/3.d0)-V**4.d0/15.d0)
      ELSE
          VR = V * VFERMI*(1.d0+1.d0/(5.d0*V*V))
      END IF

C     **** SET YR = MAXIMUM OF (VR/Z1**.67),(VRMIN/Z1**.67) OR YRMIN.
      YR = VR/Z1**.6667d0
      IF (YR .LT. YRMIN) YR = YRMIN
      A=VRMIN/Z1**.6667d0
      IF (YR .LT. A) YR = A
      A =-.803d0*YR**0.3d0+1.3167d0*YR**0.6d0+.38157d0*YR+8.983d-3*YR*YR
      IF (A .GT. 50) A = 50d0       !Prevents Underflow
      Q = 1-EXP(-A)

      IF (Q .LT. 0) THEN
        Q=0
      ELSE
        IF (Q .GT. 1) Q=1
      END IF

C     **** Q = IONIZATION LEVEL OF THE ION AT VELOCITY * YR *.
C     **** NOW WE CONVERT IONIZATION LEVEL TO EFFECTIVE CHARGE.
C====================================================================

C---------------------- Screening Distance of Ion (Lambda in B.& K.)
C     Lambda is in SCOEF(Z2,22-39), Interpolation values in SCOEF(93,22-39)
      DO J = 22 , 39                              ! Find Q Interpolation
          IF (Q .LE. SCOEF(93,J)) GO TO 10        !   in SCOEF(93,22-39)
      END DO
 10   J=J-1

      IF (J .LT. 22) THEN
        J=22
      ELSE
        IF (J .GT. 38) J = 38    ! Boilerplate
      END IF

      LAMBDA0 = SCOEF(Z1,J)
      LAMBDA1 = (Q-SCOEF(93,J))*(SCOEF(Z1,J+1)-SCOEF(Z1,J))/
     $          (SCOEF(93,J+1)-SCOEF(93,J))
      L = (LAMBDA0+LAMBDA1)/Z1**.33333d0
C====================================================================
      ZETA0 = Q+(1/(2*VFERMI**2))*(1-Q)*LOG(1+(4*L*VFERMI/1.919d0)**2)
C     **** ADD Z1**3 EFFECT
      A = LOG(E)
      IF (A .LT. 0) A = 0
      ZETA = ZETA0*(1+(1/Z1**2)*(8d-2+1.5d-3*Z2)*EXP(-(7.6d0-A)**2))
      A = VRMIN/Z1**.6667d0
      IF (A .LT. YRMIN) A = YRMIN

      IF (YR .GT. A) THEN

        CALL PSTOP(Z2,E,SP)

        SH = SP*(ZETA*Z1)**2
C Add Fermi Velocity Correction - 1995 
C     VFCORR is in SCOEF(Z2,41-54), Interpolation values in SCOEF(93,41-54)
        EION = E
        IF (EION .GT. 9999) EION = 9999         ! Not valid >1E4 keV/amu

        DO J = 41 , 53                              ! Find E Interpolation
          IF (EION .LT. SCOEF(93,J)) GO TO 20       !   in SCOEF(93,41-54)
        END DO

 20     J = J-1

        IF (J .LT. 41) THEN
          J=41
        ELSE
          IF (J .GT. 53) J = 53    ! Boilerplate
        END IF

        VFCORR0 = SCOEF(Z2,J)
        VFCORR1 = (EION-SCOEF(93,J))*(SCOEF(Z2,J+1)-SCOEF(Z2,J))/
     $            (SCOEF(93,J+1)-SCOEF(93,J))
        SH = SH*(VFCORR0+VFCORR1)
C
        RETURN
      END IF


C**** CALCULATE VELOCITY STOPPING FOR YR LESS THAN YRMIN.

      A = YRMIN*Z1**.6667d0
      IF (VRMIN .LT. A) VRMIN = A
      A = VRMIN**2-.8d0*VFERMI**2
      IF (A .LT. 0) A = 0
      VMIN = .5d0*(VRMIN+SQRT(A))
      EEE=25*VMIN**2

      CALL PSTOP(Z2,EEE,SP)

C Add Fermi Velocity Correction to Low Energy value 
C     VFCORR is in SCOEF(Z2,41-54), Interpolation values in SCOEF(93,41-54)
      EION = EEE
      IF (EION .GT. 9999) EION = 9999    ! Not valid >1E4 keV/amu

      DO J=41 , 53     ! Find E Interpolation
          IF (EION .LT. SCOEF(93,J)) GO TO 30    !   in SCOEF(93,41-54)
      END DO

 30   J=J-1

      IF (J .LT. 41) THEN
        J=41
      ELSE
        IF (J. GT. 53) J = 53    ! Boilerplate
      END IF

      VFCORR0 = SCOEF(Z2,J)
      VFCORR1 = (EION-SCOEF(93,J))*(SCOEF(Z2,J+1)-SCOEF(Z2,J))/
     $          (SCOEF(93,J+1)-SCOEF(93,J))
      SP = SP*(VFCORR0+VFCORR1)
C 
C     Following corrects for low-energy stopping, where little data exists.
C     Traditionally, this is velocity-proportional stopping, however for
C      light ions, light targets and semiconductors, a large variation exists.
C
C  Note: HIPOWER down = Se up  LAMBDA down = Se down 
      HIPOWER = .47d0       !TRIM-88 used 0.50

      IF (Z1 .EQ. 3) THEN
        HIPOWER = 0.55d0
      ELSE
        IF (Z2. LT. 7) HIPOWER = 0.375d0
      END IF

C     > Following compensates for semiconductor band-gap
      IF ((Z1 .LT. 18) .AND. ((Z2. EQ. 14) .OR. (Z2 .EQ. 32)))
     $    HIPOWER = .375d0

      SH = (SP*(ZETA*Z1)**2)*(E/EEE)**HIPOWER

      RETURN
      END


C
C PROTON ELECTRONIC STOPPING POWERS 
      SUBROUTINE PSTOP(Z2,E,SP)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/SCOEF96/ SCOEF, DPUN !, SCOEFGAS

      DOUBLE PRECISION  SCOEF(93,54),E,SP,PE,PE0,SL,SH,
     $    PPOWER,X     !,SCOEFGAS(24,11)

      INTEGER Z2, DPUN

      IF (E .LE. 1E4) THEN     !---- High Energy Stopping
C**** VELOCITY PROPORTIONAL STOPPING BELOW VELOCITY ** PE0 **.
        PE0 = 10
        PE = PE0
        IF (PE .LT. E) PE = E
        SL = (SCOEF(Z2,9)*PE**SCOEF(Z2,10))+SCOEF(Z2,11)*
     $       PE**SCOEF(Z2,12)

        SH = SCOEF(Z2,13)/PE**SCOEF(Z2,14)*LOG((SCOEF(Z2,15)/PE)+
     $       SCOEF(Z2,16)*PE)

        SP = SL*SH/(SL+SH)
        IF (E .GT. PE0) RETURN

C****   PPOWER IS THE POWER OF VELOCITY STOPPING BELOW PE0.
        PPOWER = 0.45d0     ! Low Energy Stopping: S(E)**0.45
        IF (Z2 .LT. 7) PPOWER = PPOWER-0.1d0     ! Z2=3-6 has low S(E)**0.35
        SP = SP*(E/PE0)**PPOWER
        RETURN
      END IF

      X = LOG(E)/E    !********* High Energy Stopping (6/87)
      SP = SCOEF(Z2,17)+(SCOEF(Z2,18)*X)+(SCOEF(Z2,19)*X*X)+
     $     (SCOEF(Z2,20)/X)

      RETURN
      END


C
C
      SUBROUTINE STOPCOEF
C Gets array SCOEF.95A+SCOEF.95B (92,54) 
C Gets array SCOEFGAS.95 (24,11)         
C Contains all Data and Coefficients     
C     ͻ
C     ʼ  SCOEF.95A ͹
C     > Columns 1-8 :                                      
C         (1=Atomic Number),(2=Mass of MAI),(3=Weight of MAI)
C         (4=Natural Mass),(5=Target density: g/cm3),        
C         (6=Target Density: atoms/cm3),(7=Fermi Velocity)   
C         (8=Sublimation Energy)                             
C     > Columns  9-16 : Proton stopping coefficients.      
C                      ˻                       
C     ʼ SCOEF.95B ͹
C     > Columns 1 - 4 : Proton stopping > 10 MeV.          
C     > Columns  5-13 : Ion Lambda values (width of ion)   
C     > Columns 14-28 : Tgt Fermi Velocity Correction (<5%)
C     ͼ
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      COMMON/SCOEF96/SCOEF, DPUN !, SCOEFGAS          ! SCOEF=(93,54), SCOEFGAS=(24,9)

      DOUBLE PRECISION SCOEF(93,54) !, SCOEFGAS(24,11)

      CHARACTER*1000 DUMMY

      INTEGER I,J, DPUN

      OPEN(1, FILE='SCOEF.95A', STATUS='OLD')

C     Skip Headers in file
      DO I = 1 , 2
        READ(1,*) DUMMY
      END DO

      DO I = 1 , 93
          READ(1,*) (SCOEF(I,J),J=1,16)
      END DO

      CLOSE(1)

      OPEN (1, FILE='SCOEF.95B', STATUS='OLD')

C     Skip Headers in file
      DO I = 1 , 2
        READ(1,*) DUMMY !LINE INPUT#1,A$
      END DO

      DO I = 1 , 93
        READ(1,*) (SCOEF(I,J),J=17,54)
      END DO

      CLOSE(1)

      RETURN
      END

