/* This file is part of the Allegria project
   Copyright (C) 2004 Francois Schiettekatte <francois.schiettekatte@umontreal.ca>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef CHANNELSELECTOR_H
#define CHANNELSELECTOR_H


#include <qdialog.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qpushbutton.h>

#define MAXCHAN 16
#define NUMPARAM 5  // fomula number, plus number of parameters in formula

// for a display, stores:
// - reshape parameter (camera length, t0 and E0)
// - which channel is displayed on which axis
// - graph zooming boundaries
struct SXDisplay {

	double caml;
	double delay;
	double E0;

	long paramX[NUMPARAM]; // index 0 is formula, 1 is param A, 2 is B, 3 is C
	long paramY[NUMPARAM];
	long paramE[NUMPARAM];
	long paramS[NUMPARAM];

	double xmin, xmax, ymin, ymax;

	QString name;
};


class channelSelector : public QDialog
{

	Q_OBJECT
public:
	channelSelector(long nchan, SXDisplay display, int	divide, int specsize,
		QWidget *parent=0, QString name=0 );
	~channelSelector();

	QLineEdit *spectrumSize;
	QLineEdit *spectrumDivide;

	SXDisplay getDisplay();

private:
	SXDisplay display;
	long nchan;

	QComboBox *X[NUMPARAM];
	QComboBox *Y[NUMPARAM];
	QComboBox *E[NUMPARAM];
	QComboBox *S[NUMPARAM];

	QLineEdit *camlengthEdit;
	QLineEdit *delayEdit;
	QLineEdit *E0Edit;

	QPushButton *ok, *cancel;

private slots:
	void ableTOFedit();

};

#endif // CHANNELSELECTOR_H
