/* This file is part of the Allegria project
   Copyright (C) 2004 Francois Schiettekatte <francois.schiettekatte@umontreal.ca>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef FSDATASET_H
#define FSDATASET_H

#include <qpainter.h>
#include "FSAxis.h"

#define FSDATAGROW    1024
#define MAXSYMBOL 11

class FSDataSet
{

// class data
public:
    QString legend;
    QPen linePen;			// pen used to draw lines
    QPen symbolPen;			// pen used to draw symbols

    FSDataSet * prevDataSet;            // a pointer to the previous dataset related to a same graph (NULL means no previous, i.e. first)
    FSDataSet * nextDataSet;            // a pointer to the next dataset related to a same graph (NULL means no next, i.e. last)

    double * xp;			// pointer to x data
    double * yp;			// pointer to y data

    QRect legendRect;       // rectangle in which the legend of this data set is displayed

    bool dataHaveChanged;		// data need to be saved 

private:
    long ndata;				// number of datapoint
    long maxdata;			// number available space for data

    double xMin, xMax, yMin, yMax;  	// extrema of dataset
    double xMingt0, yMingt0;  		// minimum value greater than 0
	
    int symbol;
    double symbolSize;

// constructors
public:
    FSDataSet();
    FSDataSet(FSDataSet * dataset);
    FSDataSet(QString llegend, QPen llinePen = QPen(), int ssymbol = 0, double ssymbolSize = 5, QPen ssymbolPen = QPen());
   ~FSDataSet();

// methods
private:
    void _init();

public:
    void append(double xx, double yy);
    void grow(int n);
    void clear();
    bool loadfile(QString filename, int colx = 1, int coly = 2);     // col1 <= 0 means x[i]=i; col2 <= 0 means y[i]=i;
	bool savefile(QString filename, bool saveAsInt = false);
    bool swallow(QString filename);

    int interpolate(int n, double xx[], double yy[], double outVal);
    bool extrema();
    void sort();
    void swap();

    void draw(QPainter *dc, FSAxis *xax, FSAxis *yax, double fSize );
    void drawSymbol(QPainter *dc, QPointArray *p, int count, double fSize );

// properties
public:
    int getndata(){ return ndata; }

    double getxMin(){ return xMin; }
    double getxMax(){ return xMax; }
    double getyMin(){ return yMin; }
    double getyMax(){ return yMax; }
    double getxMingt0(){ return xMingt0; }
    double getyMingt0(){ return yMingt0; }

    int getsymbol(){ return symbol; }
    bool setsymbol(int n){ if(n>=0&&n<=MAXSYMBOL) symbol=n; return (n>=0&&n<=MAXSYMBOL); }

    double getsymbolSize(){ return symbolSize; }
    bool setsymbolSize(double n){ if(n>=0.) symbolSize=n; return (n>=0.); }

    QString getlegend(){ return QString(dataHaveChanged?"*":"")+legend; }
    QString getname(){ return legend; }

    bool edit( void );

};

#endif // FSDATASET_H
