#ifndef SPLINE_H
#define SPLINE_H

//=============================================================================
// Description: Cette fonction determine la valeur des 4 segments
//              de splines cubiques a la position rx sur la
//              maille r constituee de ngrid noeuds.
//
// Auteur:      Inconnu, adaptation par Francois Schiettekatte
//
// Entree:      ngrid: nombre de noeuds pour la definition de la maille
//                 rx: point auquel il faut evaluer les 4 segments de splines
//
// Sortie:      s[4]: valeur des 4 sections de spline
//              valeur de retour >=  0: interval ou se situe rx
//                                = -1: ngrid < 4
//                                = -2: rx hors de l'interval
//=============================================================================
int spline(int ngrid, double rx, double * s)
{
  int i;
  int m[5];
  double *r;
  r = new double[ngrid];

  if(ngrid<4)
    return -1;

  if(rx < 0. || rx > 1.)
    return -2;

  for(i=0; i<ngrid; i++)
    r[i] = double(i) / double(ngrid-1.);

  // define in which segment is rx
  for(i=1; i<ngrid; i++)
    if(rx<r[i])
        break;
  i--;

  //4 spline in each place, 4th segment of spline i,   3rd segment of spline i+1,
  //                        2nd segment of spline i+2, 1st segment of spline i+3,
  for(int j=0; j<4; j++)
  {
     int l=0;
	 m[0]=i-3+j;
     if(m[0]<0)
        m[0]=0;

	 m[1]=i-2+j;
     if(m[1]<0)
        m[1]=0;

	 m[2]=i-1+j;
     if(m[2]<0)
        m[2]=0;

     if(m[2]>=ngrid) {
         m[2] = m[1]; l = 4;
     }

     m[3]=i+j;
     if(m[3]>=ngrid) {
         m[3] = m[2];
         l = l?l:3;
     }

     m[4]=i+1+j;
     if(m[4]>=ngrid) {
         m[4] = m[3];
         l = l?l:2;
     }

     double c[4], p[4]= {0.,0.,0.,0.};
     double c1, c2, c3, c4;

	 switch (l) {
	   case 0:
	     break;
	   case 2:
	     c4 = 12./(r[m[3]]-r[m[0]])/(r[m[3]]-r[m[1]])/(r[m[3]]-r[m[2]]);
	     c3 = -4./(r[m[3]]-r[m[1]])/(r[m[3]]-r[m[2]])/(r[m[3]]-r[m[0]])/(r[m[3]]-r[m[0]])
		  -4./(r[m[3]]-r[m[0]])/(r[m[3]]-r[m[2]])/(r[m[3]]-r[m[1]])/(r[m[3]]-r[m[1]])
		  -4./(r[m[3]]-r[m[0]])/(r[m[3]]-r[m[1]])/(r[m[3]]-r[m[2]])/(r[m[3]]-r[m[2]]);

	     p[3] =                    -c3;
	     p[2] =     c4          +3.*c3*r[m[3]];
	     p[1] = -2.*c4          -3.*c3*r[m[3]]*r[m[3]];
	     p[0] = c4*r[m[3]]*r[m[3]] +c3*r[m[3]]*r[m[3]]*r[m[3]];
	     break;
	   case 3:
	     c4 =  12./(r[m[2]]-r[m[0]])/(r[m[2]]-r[m[1]]);
	     c3 = -12./(r[m[2]]-r[m[1]])/(r[m[2]]-r[m[0]])/(r[m[2]]-r[m[0]])
		  -12./(r[m[2]]-r[m[0]])/(r[m[2]]-r[m[1]])/(r[m[2]]-r[m[1]]);
	     c2 =   4./(r[m[2]]-r[m[1]])/(r[m[2]]-r[m[0]])/(r[m[2]]-r[m[0]])/(r[m[2]]-r[m[0]])
		  + 4./(r[m[2]]-r[m[0]])/(r[m[2]]-r[m[0]])/(r[m[2]]-r[m[1]])/(r[m[2]]-r[m[1]])
		  + 4./(r[m[2]]-r[m[0]])/(r[m[2]]-r[m[1]])/(r[m[2]]-r[m[1]])/(r[m[2]]-r[m[1]]);

	     p[3] =                               -c2;
	     p[2] =             c3             +3.*c2*r[m[2]];
	     p[1] = -c4     -2.*c3*r[m[2]]     -3.*c2*r[m[2]]*r[m[2]];
	     p[0] =  c4*r[m[2]]+c3*r[m[2]]*r[m[2]]+c2*r[m[2]]*r[m[2]]*r[m[2]];
	     break;
	   case 4:
	     c4 =   4./(r[m[1]]-r[m[0]]);
	     c3 = -12./(r[m[1]]-r[m[0]])/(r[m[1]]-r[m[0]]);
	     c2 =  12./(r[m[1]]-r[m[0]])/(r[m[1]]-r[m[0]])/(r[m[1]]-r[m[0]]);
	     c1 = - 4./(r[m[1]]-r[m[0]])/(r[m[1]]-r[m[0]])/(r[m[1]]-r[m[0]])/(r[m[1]]-r[m[0]]);

	     p[3] =                                  -c1;
	     p[2] =                c2             +3.*c1*r[m[1]];
	     p[1] =    -c3     -2.*c2*r[m[1]]     -3.*c1*r[m[1]]*r[m[1]];
	     p[0] = c4 +c3*r[m[1]]+c2*r[m[1]]*r[m[1]]+c1*r[m[1]]*r[m[1]]*r[m[1]];
	     break;
	 }

	 for(int k=l; k<=j; k++) {
	   c[3-k]=4.0;
	   for(int k2=0; k2<4; k2++)
	     c[3-k] /= r[m[4-k]]-r[m[4-(k>k2?k2:k2+1)]];

       double rm = r[m[4-k]];
       double cc = k>3?0.:*(c+3-k);
	   p[3] +=     -cc;
	   p[2] += +3.0*cc*rm;
	   p[1] += -3.0*cc*rm*rm;
	   p[0] +=      cc*rm*rm*rm;
	 }
	 s[j] = p[0] + rx*(p[1] + rx*(p[2] + rx*p[3]));
	 if(s[j] < 0.) s[j] = 0.;
  }

  delete r;

  return i; // return the place of rx within the r[] mesh
}


#endif //SPLINE_H

